#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from six import string_types
from libsignetsim.model.math.MathSymbol import MathSymbol
from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.sympy_shortcuts import SympySymbol
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs


class MathVariable(object):

	# C code variable type
	VAR_ODE = 0
	VAR_ASS = 1
	VAR_CST = 2
	VAR_DAE = 3

	def __init__(self, model, is_from_reaction=None):

		self.__model = model

		self.type = None
		self.ind = None

		self.symbol = MathSymbol(self.__model, self, is_from_reaction)

		self.value = MathFormula(self.__model)
		self.derivative_value = MathFormula(self.__model)

		self.isInitialized = False
		self.isDerivativeInitialized = False
		self.constant = None
		self.boundaryCondition = False
		self.is_in_dae = None
		self.isFromReaction = is_from_reaction

		self.mathVariable = False
		# self.mathIsConcentration = None
		# self.mathCompartment = None

	def new(self, string=None):

		if self.isFromReaction is not None:
			self.symbol.setInternalMathFormula(SympySymbol("_local_%d_%s" % (self.isFromReaction.objId, self.getSbmlId())))
		else:
			self.symbol.setInternalMathFormula(SympySymbol(self.getSbmlId()))

		self.setValue(0)
		self.setDerivativeValue(0)

	def copy(self, obj, symbols_subs={}, conversion_factor=None, pure_math_variable=False):

		if obj.symbol.getSymbol() in list(symbols_subs.keys()):
			self.symbol.setInternalMathFormula(symbols_subs[obj.symbol.getSymbol()])
		else:
			self.symbol.setInternalMathFormula(obj.symbol.getSymbol())

		# self.symbol.setInternalMathFormula(SympySymbol(prefix + str(obj.symbol.getSymbol())))

		if obj.value is not None and obj.value.getInternalMathFormula() is not None:
			t_formula = unevaluatedSubs(obj.value.getInternalMathFormula(), symbols_subs)
			# t_formula = unevaluatedSubs(t_formula, replacements)

			if conversion_factor is not None:
				t_formula *= conversion_factor
			self.value.setInternalMathFormula(t_formula)

		if obj.derivative_value is not None and obj.derivative_value.getInternalMathFormula() is not None:
			t_formula = unevaluatedSubs(obj.derivative_value.getInternalMathFormula(), symbols_subs)
			# t_formula = unevaluatedSubs(t_formula, replacements)

			if conversion_factor is not None:
				t_formula *= conversion_factor

			self.derivative_value.setInternalMathFormula(t_formula)

		self.isInitialized = obj.isInitialized
		self.isDerivativeInitialized = obj.isDerivativeInitialized
		self.constant = obj.constant
		self.is_in_dae = obj.is_in_dae
		self.boundaryCondition = obj.boundaryCondition
		self.ind = obj.ind
		self.type = obj.type

		if pure_math_variable:
			self.mathVariable = True
		# 	if not obj.mathVariable:
		# 		if obj.isSpecies() and obj.isConcentration():
		# 			self.mathIsConcentration = True
		# 			self.mathCompartment = obj.getCompartment().symbol.getInternalMathFormula()
		# 	else:
		# 		self.mathIsConcentration = obj.mathIsConcentration
		# 		self.mathCompartment = obj.mathCompartment

	def readSbml(self, sbml_variable, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		self.readSbmlVariable(sbml_variable, sbml_level, sbml_version)


	def writeSbml(self, sbml_variable, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		self.writeSbmlVariable(sbml_variable, sbml_level, sbml_version)

	def getValue(self):
		if self.isInitialized:
			return self.value.getValueMathFormula()

	def setValue(self, value):
		if value is None:
			self.isInitialized = False
			self.value.setInternalMathFormula(None)

		elif isinstance(value, int) or isinstance(value, float):
			self.isInitialized = True
			self.value.setValueMathFormula(value)

		elif isinstance(value, string_types):
			self.isInitialized = True
			self.value.setPrettyPrintMathFormula(value)

	def setDerivativeValue(self, value):
		if isinstance(value, int) or isinstance(value, float):
			self.derivative_value.setValueMathFormula(value)

	def getSbmlValue(self, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		return self.value.getSbmlMathFormula(sbml_level, sbml_version)

	def setSbmlValue(self, sbml_value, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		return self.value.setSbmlMathFormula(sbml_value, sbml_level, sbml_version)


	def getSbmlSymbol(self, sbml_level=Settings.defaultSbmlLevel, sbml_version=Settings.defaultSbmlVersion):
		return self.symbol.getSbmlMathFormula(sbml_level, sbml_version)

	def getCValue(self):

		if (self.isSpecies() and (self.isConcentration() or self.isDeclaredConcentration)) or self.isReaction():
			return "RCONST(0.0)"

		elif self.value.getInternalMathFormula() is None:
			if self.isCompartment() or self.isParameter():
				return "RCONST(1.0)"
			else:
				return "RCONST(0.0)"

		return self.value.getCMathFormula()


	def getDerivativeCValue(self):

		if (self.isSpecies() and self.isDeclaredConcentration) or self.isReaction() or self.derivative_value.getInternalMathFormula() is None:
			return "RCONST(0.0)"

		return self.derivative_value.getCMathFormula()


	def isDerivative(self):
		return self.type == self.VAR_ODE


	def isAssignment(self):
		return self.type == self.VAR_ASS


	def isConstant(self):
		return self.type == self.VAR_CST

	def isAlgebraic(self):
		return self.type == self.VAR_DAE


	def getPos(self):

		if self.type == self.VAR_ODE:
			return self.ind

		elif self.type == self.VAR_DAE:
			return self.__model.nbOdes + self.ind
		elif self.type == self.VAR_ASS:
			return self.__model.nbOdes + self.__model.nbAlgebraics + self.ind

		elif self.type == self.VAR_CST:
			return self.__model.nbOdes + self.__model.nbAlgebraics + self.__model.nbAssignments + self.ind


	# def getODE(self, including_fast_reactions=None, math_type=MathFormula.MATH_INTERNAL, forcedConcentration=False, symbols=False, rawFormula=True):
	#
	# 	if self.isRateRuled():
	# 		return self.isRuledBy().getDefinition(forcedConcentration).getMathFormula(math_type)
	#
	# 	else:
	# 		return MathFormula.ZERO
	#
	# def getODE(self, including_fast_reactions=True, forcedConcentration=False):
	#
	# 	if self.isRateRuled():
	# 		return self.isRuledBy().getDefinition(forcedConcentration)
	#
	# 	else:
	# 		t_formula = MathFormula(self.__model)
	# 		t_formula.setInternalMathFormula(MathFormula.ZERO)
	# 		return t_formula


	def getODE(self, including_fast_reactions=True, rawFormula=False):

		if self.isRateRuled() and self.isRuledBy().isValid():
			return self.isRuledBy().getDefinition(rawFormula=rawFormula)

		else:
			t_formula = MathFormula(self.__model)
			t_formula.setInternalMathFormula(MathFormula.ZERO)
			return t_formula

	def setSbmlId(self, sbml_id, prefix=""):
		self.symbol.setValueMathFormula(prefix+sbml_id)

	def renameSymbol(self, old_sbml_id, new_sbml_id):
		self.symbol.renameSbmlId(old_sbml_id, new_sbml_id)

	def renameSbmlIdInValue(self, old_sbml_id, new_sbml_id):
		self.value.renameSbmlId(old_sbml_id, new_sbml_id)

	def renameSbmlId(self, old_sbml_id, new_sbml_id):
		self.value.renameSbmlId(old_sbml_id, new_sbml_id)
		self.symbol.renameSbmlId(old_sbml_id, new_sbml_id)

	def isInRules(self):
		return self.__model.listOfRules.containsVariable(self)

	def isInReactionsRates(self):
		return self.__model.listOfReactions.containsVariable(self)

	def isConcentration(self):
		""" There *IS* a case where a species doesn't have compartment I guess,
			so we actually need to check before looking at the spatialDimensions
		"""

		return (not self.mathVariable and self.isSpecies()
				and (self.getCompartment() is not None and not self.getCompartment().spatialDimensions == 0)
				and not self.hasOnlySubstanceUnits)

	def getSymbol(self):

		return self.symbol

	def getSymbolStr(self):

		return self.symbol.getSymbol().name


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import division

from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.sbml.SimpleSbmlObject import SimpleSbmlObject
from libsignetsim.settings.Settings import Settings
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs


class EventPriority(SimpleSbmlObject, MathFormula):
	""" Events priority's definition """

	def __init__(self, model, math_only=False):

		self.__model = model
		MathFormula.__init__(self, model, MathFormula.MATH_PRIORITY)

		self.mathOnly = math_only
		if not self.mathOnly:
			SimpleSbmlObject.__init__(self, model)

	def readSbml(self, sbml_priority,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Reads an event priority definition from a sbml file """

		SimpleSbmlObject.readSbml(self, sbml_priority, sbml_level, sbml_version)
		MathFormula.readSbml(self, sbml_priority.getMath(), sbml_level, sbml_version)


	def writeSbml(self, sbml_event,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Writes an event priority definition to a sbml file """

		sbml_priority = sbml_event.createPriority()
		SimpleSbmlObject.writeSbml(self, sbml_priority, sbml_level, sbml_version)
		sbml_priority.setMath(MathFormula.writeSbml(self, sbml_level, sbml_version))


	def copy(self, obj, symbols_subs={}, conversion_factors={}):

		if not self.mathOnly:
			SimpleSbmlObject.copy(self, obj)

		t_convs = {}
		for var, conversion in list(conversion_factors.items()):
			t_convs.update({var: var/conversion})

		t_formula = unevaluatedSubs(obj.getInternalMathFormula(rawFormula=False), symbols_subs)
		t_formula = unevaluatedSubs(t_formula, t_convs)
		MathFormula.setInternalMathFormula(self, t_formula)



	def copySubmodel(self, obj):
		MathFormula.setInternalMathFormula(self, obj.getDeveloppedInternalMathFormula())




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import absolute_import

# from builtins import str


from libsignetsim.settings.Settings import Settings
from libsignetsim.sedml.SedmlException import SedmlMathException
import libsbml
# token: cn , ci , csymbol , sep
from libsedml import AST_NAME, AST_NAME_TIME, AST_INTEGER, AST_REAL
from .sympy_shortcuts import SympyInteger, SympyFloat, SympySymbol
# qualifiers: degree , bvar , logbase

# general : apply , piecewise , piece , otherwise , lambda
from libsedml import AST_FUNCTION_PIECEWISE, AST_LAMBDA
from .sympy_shortcuts import SympyPiecewise, SympyITE, SympyLambda

# relational operators: eq , neq , gt , lt , geq , leq
from libsedml import (
	AST_RELATIONAL_EQ, AST_RELATIONAL_NEQ, AST_RELATIONAL_GT, AST_RELATIONAL_LT, AST_RELATIONAL_GEQ, AST_RELATIONAL_LEQ
)
from .sympy_shortcuts import (
	SympyEqual, SympyUnequal, SympyStrictGreaterThan, SympyStrictLessThan, SympyGreaterThan, SympyLessThan
)


# arithmetic operators: plus , minus , times , divide , power , root , abs , exp , ln , log , floor , ceiling ,
from libsedml import (
	AST_PLUS, AST_MINUS, AST_TIMES, AST_DIVIDE, AST_FUNCTION_POWER, AST_FUNCTION_ROOT, AST_FUNCTION_ABS, AST_FUNCTION_EXP,
	AST_FUNCTION_LN, AST_FUNCTION_LOG, AST_FUNCTION_FLOOR, AST_FUNCTION_CEILING
)
from .sympy_shortcuts import (SympyAdd, SympyMul, SympyPow, SympyAbs, SympyExp, SympyLog, SympyFloor, SympyCeiling)


# factorial
from libsedml import AST_FUNCTION_FACTORIAL
from .sympy_shortcuts import SympyFactorial

# logical operators: and , or , xor , not
from libsedml import AST_LOGICAL_AND, AST_LOGICAL_OR, AST_LOGICAL_XOR, AST_LOGICAL_NOT
from .sympy_shortcuts import SympyAnd, SympyOr, SympyXor, SympyNot

# trigonometric operators: sin , cos , tan , sec , csc , cot
from libsedml import (
	AST_FUNCTION_SIN, AST_FUNCTION_COS, AST_FUNCTION_TAN, AST_FUNCTION_SEC, AST_FUNCTION_CSC, AST_FUNCTION_COT
)
from .sympy_shortcuts import (SympySin, SympyCos, SympyTan, SympySec, SympyCsc, SympyCot)


# trigonometric operators: sinh , cosh , tanh , sech , csch , coth ,
from libsedml import (AST_FUNCTION_SINH, AST_FUNCTION_COSH, AST_FUNCTION_TANH, AST_FUNCTION_COTH)
from .sympy_shortcuts import (SympySinh, SympyCosh, SympyTanh, SympyCoth)

# trigonometric operators: arcsin , arccos , arctan , arcsec , arccsc , arccot
from libsedml import (
	AST_FUNCTION_ARCSIN, AST_FUNCTION_ARCCOS, AST_FUNCTION_ARCTAN,
	AST_FUNCTION_ARCSEC, AST_FUNCTION_ARCCSC, AST_FUNCTION_ARCCOT
)
from .sympy_shortcuts import (SympyAsin, SympyAcos, SympyAtan, SympyAsec, SympyAcsc, SympyAcot)

# trigonometric operators: arcsinh , arccosh , arctanh , arcsech , arccsch , arccoth
from libsedml import (
	AST_FUNCTION_ARCSINH, AST_FUNCTION_ARCCOSH, AST_FUNCTION_ARCTANH,
	AST_FUNCTION_ARCSECH, AST_FUNCTION_ARCCSCH, AST_FUNCTION_ARCCOTH
)
from .sympy_shortcuts import (SympyAsinh, SympyAcosh, SympyAtanh, SympyAcoth)

from .sympy_shortcuts import (SympyUndefinedFunction)
# constants: true , false , notanumber , pi , infinity , exponentiale
# TODO : Infinity, NaN
from libsedml import (AST_CONSTANT_TRUE, AST_CONSTANT_FALSE, AST_CONSTANT_PI, AST_CONSTANT_E)
from .sympy_shortcuts import (
	SympyTrue, SympyFalse, SympyPi, SympyE, SympyExp1, SympyInf, SympyNan, SympyOne, SympyNegOne, SympyHalf, SympyZero
)
# import libsbml
from libsedml import ASTNode, parseFormula, formulaToString
from six.moves import reload_module
reload_module(libsbml)


class SedmlMathWriter(object):
	""" Class for handling math formulaes """

	def __init__(self, document):
		""" Constructor """

		self.__document = document

	def translateForSedml(self, tree, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):
		""" Translate a sympy tree into a C string """

		if tree.func == SympySymbol:
			# if str(tree) == '_time_':
			# 	t_ast = ASTNode()
			# 	t_ast.setType(AST_NAME_TIME)
			# 	t_ast.setName("time")
			# 	return t_ast
			#
			# else:
			t_ast = ASTNode()
			t_ast.setType(AST_NAME)
			t_ast.setName(tree.name)
			return t_ast

		elif tree.func == SympyInteger:
			t_ast = ASTNode()
			t_ast.setType(AST_INTEGER)
			t_ast.setValue(int(tree))
			return t_ast

		elif tree.func == SympyFloat:
			t_ast = ASTNode()
			t_ast.setType(AST_REAL)
			t_ast.setValue(float(tree))
			return t_ast

		elif tree.func == SympyNegOne:
			t_ast = ASTNode()
			t_ast.setType(AST_INTEGER)
			t_ast.setValue(-1)
			return t_ast

		elif tree.func == SympyOne:
			t_ast = ASTNode()
			t_ast.setType(AST_INTEGER)
			t_ast.setValue(1)
			return t_ast

		elif tree.func == SympyHalf:
			t_ast = ASTNode()
			t_ast.setType(AST_REAL)
			t_ast.setValue(0.5)
			return t_ast

		elif tree.func == SympyZero:
			t_ast = ASTNode()
			t_ast.setType(AST_INTEGER)
			t_ast.setValue(0)
			return t_ast

		elif tree == SympyPi:
			t_ast = ASTNode()
			t_ast.setType(AST_CONSTANT_PI)
			return t_ast

		elif tree.func == SympyE or tree.func == SympyExp1:
			t_ast = ASTNode()
			t_ast.setType(AST_CONSTANT_E)
			return t_ast

		elif tree == SympyInf:
			t_ast = ASTNode()
			t_ast.setType(AST_REAL)
			t_ast.setValue(float("inf"))
			return t_ast

		elif tree == -SympyInf:
			t_ast = ASTNode()
			t_ast.setType(AST_REAL)
			t_ast.setValue(float("-inf"))
			return t_ast

		elif tree == SympyNan:
			t_ast = ASTNode()
			t_ast.setType(AST_REAL)
			t_ast.setValue(float("nan"))
			return t_ast

		elif tree == SympyTrue or tree == True:
			t_ast = ASTNode()
			t_ast.setType(AST_CONSTANT_TRUE)
			return t_ast

		elif tree == SympyFalse or tree == False:
			t_ast = ASTNode()
			t_ast.setType(AST_CONSTANT_FALSE)
			return t_ast

		elif tree.func == SympyAdd:

			t_ast = ASTNode()
			t_ast.setType(AST_PLUS)
			for i_arg, arg in enumerate(tree.args):
				t_ast.addChild(self.translateForSedml(arg, level, version))

			return t_ast

		elif tree.func == SympyMul:

			if len(tree.args) == 2:
				if tree.args[0].func == SympyNegOne:
					t_ast = ASTNode()
					t_ast.setType(AST_MINUS)
					t_ast.addChild(self.translateForSedml(tree.args[1], level, version))
					return t_ast

				if tree.args[1].func == SympyNegOne:
					t_ast = ASTNode()
					t_ast.setType(AST_MINUS)
					t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
					return t_ast

			t_ast = ASTNode()
			t_ast.setType(AST_TIMES)
			t_tree_denominator = None
			# t_tree_numerator = None
			# for i_arg, arg in enumerate(tree.args):
			# 	if arg.func == SympyPow and arg.args[1].func == SympyNegOne:
			#
			# 		if t_tree_denominator is None:
			# 			t_tree_denominator = self.translateForSedml(arg.args[0], level, version)
			# 		elif t_tree_denominator.getType() != AST_TIMES:
			# 			t_tree_denominator_new = ASTNode()
			# 			t_tree_denominator_new.setType(AST_TIMES)
			# 			t_tree_denominator_new.addChild(t_tree_denominator)
			# 			t_tree_denominator_new.addChild(self.translateForSedml(arg.args[0], level, version))
			# 			t_tree_denominator = t_tree_denominator_new
			# 		else:
			# 			t_tree_denominator.addChild(self.translateForSedml(arg.args[0], level, version))
			# 	else:
			# 		t_tree_numerator = self.translateForSedml(arg, level, version)
			# 		t_ast.addChild(t_tree_numerator)
			#
			# if t_tree_denominator is not None:
			# 	t_ast_2 = ASTNode()
			# 	t_ast_2.setType(AST_DIVIDE)
			# 	if t_ast.getNumChildren() == 1:
			# 		t_ast_2.addChild(t_tree_numerator)
			# 	else:
			# 		t_ast_2.addChild(t_ast)
			# 	t_ast_2.addChild(t_tree_denominator)
			# 	# print formulaToString(t_ast_2)
			# 	return t_ast_2
			# # print formulaToString(t_ast)
			# return t_ast
			t_tree_numerator = None
			for i_arg, arg in enumerate(tree.args):
				if arg.func == SympyPow and arg.args[1].func == SympyNegOne:

					if t_tree_denominator is None:
						t_tree_denominator = self.translateForSedml(arg.args[0], level, version)
					elif t_tree_denominator.getType() != AST_TIMES:
						t_tree_denominator_new = ASTNode()
						t_tree_denominator_new.setType(AST_TIMES)
						t_tree_denominator_new.addChild(t_tree_denominator)
						t_tree_denominator_new.addChild(self.translateForSedml(arg.args[0], level, version))
						t_tree_denominator = t_tree_denominator_new
					else:
						t_tree_denominator.addChild(self.translateForSedml(arg.args[0], level, version))
				else:
					t_tree_numerator = self.translateForSedml(arg, level, version)
					t_ast.addChild(self.translateForSedml(arg, level, version))
					# t_ast.addChild(self.translateForSedml(arg, level, version))

			if t_tree_denominator is not None:
				t_ast_2 = ASTNode()
				t_ast_2.setType(AST_DIVIDE)
				if t_ast.getNumChildren() == 1:
					t_ast_2.addChild(t_tree_numerator)
				else:
					t_ast_2.addChild(t_ast)
				t_ast_2.addChild(t_tree_denominator)
				# print formulaToString(t_ast_2)
				return t_ast_2
			# print formulaToString(t_ast)
			return t_ast


		# AST_FUNCTION_ABS
		elif tree.func == SympyAbs:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_ABS)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_ARCCOS
		elif tree.func == SympyAcos:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_ARCCOS)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_ARCCOSH
		elif tree.func == SympyAcosh:
			if tree.args[0].func == SympyPow and tree.args[0].args[1] == SympyInteger(-1):
				t_ast = ASTNode()
				t_ast.setType(AST_FUNCTION_ARCSECH)
				t_ast.addChild(self.translateForSedml(tree.args[0].args[0], level, version))
				return t_ast

			else:
				t_ast = ASTNode()
				t_ast.setType(AST_FUNCTION_ARCCOSH)
				t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
				return t_ast

		# AST_FUNCTION_ARCCOT
		elif tree.func == SympyAcot:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_ARCCOT)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_ARCCOTH
		elif tree.func == SympyAcoth:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_ARCCOTH)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_ARCSIN
		elif tree.func == SympyAsin:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_ARCSIN)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_ARCSINH
		elif tree.func == SympyAsinh:
			if tree.args[0].func == SympyPow and tree.args[0].args[1] == SympyInteger(-1):
				t_ast = ASTNode()
				t_ast.setType(AST_FUNCTION_ARCCSCH)
				t_ast.addChild(self.translateForSedml(tree.args[0].args[0], level, version))
				return t_ast
			else:
				t_ast = ASTNode()
				t_ast.setType(AST_FUNCTION_ARCSINH)
				t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
				return t_ast

		# AST_FUNCTION_ARCTAN
		elif tree.func == SympyAtan:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_ARCTAN)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_ARCTANH
		elif tree.func == SympyAtanh:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_ARCTANH)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_ARCSEC
		elif tree.func == SympyAsec:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_ARCSEC)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_ARCCSC
		elif tree.func == SympyAcsc:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_ARCCSC)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_CEILING
		elif tree.func == SympyCeiling:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_CEILING)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_COS
		elif tree.func == SympyCos:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_COS)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_COSH
		elif tree.func == SympyCosh:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_COSH)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_COT
		elif tree.func == SympyCot:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_COT)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_COTH
		elif tree.func == SympyCoth:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_COTH)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_CSC
		elif tree.func == SympyCsc:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_CSC)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_DELAY
		# TODO
		#
		# AST_FUNCTION_EXP
		elif tree.func == SympyExp:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_EXP)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_FACTORIAL
		elif tree.func == SympyFactorial:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_FACTORIAL)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_FLOOR
		elif tree.func == SympyFloor:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_FLOOR)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_LOG
		#TODO
		elif tree.func == SympyLog:
			if len(tree.args) < 2:
				t_ast = ASTNode()
				t_ast.setType(AST_FUNCTION_LN)
				t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
				return t_ast

			else:
				if tree.args[1] == SympyInteger(10):
					t_ast = ASTNode()
					t_ast.setType(AST_FUNCTION_LOG)

					t_ast_2 = ASTNode()
					t_ast_2.setType(AST_INTEGER)
					t_ast_2.setValue(10)

					t_ast.addChild(t_ast_2)
					t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
					return t_ast
				else:
					t_ast = ASTNode()
					t_ast.setType(AST_FUNCTION_LOG)
					t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
					t_ast.addChild(self.translateForSedml(tree.args[1], level, version))
					return t_ast

		# AST_FUNCTION_PIECEWISE
		elif tree.func == SympyPiecewise:

			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_PIECEWISE)

			(t_val, t_cond) = tree.args[0]

			t_ast.addChild(self.translateForSedml(t_val, level, version))
			t_ast.addChild(self.translateForSedml(t_cond, level, version))
			last_child = []
			for piece in range(1, len(tree.args)):
				(t_val, t_cond) = tree.args[piece]

				if piece == (len(tree.args)-1):
					t_ast.addChild(self.translateForSedml(t_val, level, version))
				else:
					t_ast.addChild(self.translateForSedml(t_val, level, version))
					t_ast.addChild(self.translateForSedml(t_cond, level, version))

			return t_ast

		elif tree.func == SympyITE:

			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_PIECEWISE)

			t_cond = tree.args[0]
			t_val = tree.args[1]
			t_val_else = tree.args[2]
			t_ast.addChild(self.translateForSedml(t_val, level, version))
			t_ast.addChild(self.translateForSedml(t_cond, level, version))
			t_ast.addChild(self.translateForSedml(t_val_else, level, version))
			return t_ast

		#TODO
		# AST_FUNCTION_POWER
		elif tree.func == SympyPow:

			# A standard 1/b division will be written mul(1, pow(b,-1))
			# We recognize it and write it as a division
			# Important to note that a/b is written mul(a, mul(1, pow(b,-1)))
			# so we don't need to look for a
			if tree.args[1].func == SympyNegOne:
				t_ast = ASTNode()
				t_ast.setType(AST_DIVIDE)

				t_ast_2 = ASTNode()
				t_ast_2.setType(AST_INTEGER)
				t_ast_2.setValue(1)

				t_ast.addChild(t_ast_2)
				t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
				return t_ast

			# Similar for SQRT(x), it's written pow(x, pow(2, -1))
			elif (
					tree.args[1].func == SympyPow
					and tree.args[1].args[0] == SympyInteger(2)
					and tree.args[1].args[1].func == SympyNegOne
			):
				t_ast = ASTNode()
				t_ast.setType(AST_FUNCTION_ROOT)

				t_ast_2 = ASTNode()
				t_ast_2.setType(AST_INTEGER)
				t_ast_2.setValue(2)

				t_ast.addChild(t_ast_2)
				t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
				return t_ast

			# print "\n"
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_POWER)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			t_ast.addChild(self.translateForSedml(tree.args[1], level, version))
			return t_ast

		# AST_FUNCTION_SEC
		elif tree.func == SympySec:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_SEC)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_SIN
		elif tree.func == SympySin:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_SIN)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_SINH
		elif tree.func == SympySinh:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_SINH)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_TAN
		elif tree.func == SympyTan:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_TAN)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast

		# AST_FUNCTION_TANH
		elif tree.func == SympyTanh:
			t_ast = ASTNode()
			t_ast.setType(AST_FUNCTION_TANH)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast






		elif tree.func == SympyEqual:
			t_ast = ASTNode()
			t_ast.setType(AST_RELATIONAL_EQ)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			t_ast.addChild(self.translateForSedml(tree.args[1], level, version))
			return t_ast

		elif tree.func == SympyUnequal:
			t_ast = ASTNode()
			t_ast.setType(AST_RELATIONAL_NEQ)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			t_ast.addChild(self.translateForSedml(tree.args[1], level, version))
			return t_ast

		elif tree.func == SympyGreaterThan:
			t_ast = ASTNode()
			t_ast.setType(AST_RELATIONAL_GEQ)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			t_ast.addChild(self.translateForSedml(tree.args[1], level, version))
			return t_ast

		elif tree.func == SympyLessThan:
			t_ast = ASTNode()
			t_ast.setType(AST_RELATIONAL_LEQ)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			t_ast.addChild(self.translateForSedml(tree.args[1], level, version))
			return t_ast

		elif tree.func == SympyStrictGreaterThan:
			t_ast = ASTNode()
			t_ast.setType(AST_RELATIONAL_GT)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			t_ast.addChild(self.translateForSedml(tree.args[1], level, version))
			return t_ast

		elif tree.func == SympyStrictLessThan:
			t_ast = ASTNode()
			t_ast.setType(AST_RELATIONAL_LT)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			t_ast.addChild(self.translateForSedml(tree.args[1], level, version))
			return t_ast

		elif tree.func == SympyAnd:
			t_ast = ASTNode()
			t_ast.setType(AST_LOGICAL_AND)
			for i_arg in range(0, len(tree.args)):
				t_ast.addChild(self.translateForSedml(tree.args[i_arg], level, version))
			return t_ast

		elif tree.func == SympyOr:
			t_ast = ASTNode()
			t_ast.setType(AST_LOGICAL_OR)
			for i_arg in range(0, len(tree.args)):
				t_ast.addChild(self.translateForSedml(tree.args[i_arg], level, version))
			return t_ast

		elif tree.func == SympyXor:
			t_ast = ASTNode()
			t_ast.setType(AST_LOGICAL_XOR)
			for i_arg in range(0, len(tree.args)):
				t_ast.addChild(self.translateForSedml(tree.args[i_arg], level, version))
			return t_ast

		elif tree.func == SympyNot:
			# print tree
			t_ast = ASTNode()
			t_ast.setType(AST_LOGICAL_NOT)
			t_ast.addChild(self.translateForSedml(tree.args[0], level, version))
			return t_ast
		elif tree.func == SympyLambda:
			t_ast = ASTNode()
			t_ast.setType(AST_LAMBDA)
			t_args = list(tree.args[0])
			for arg in t_args:
				t_ast.addChild(self.translateForSedml(arg, level, version))
			t_ast.addChild(self.translateForSedml(tree.args[1], level, version))
			return t_ast

		elif isinstance(tree.func, SympyUndefinedFunction):
			if str(tree.func) == "min":
				return parseFormula("min(" + formulaToString(self.translateForSedml(tree.args[0], level, version)) + ")")
			elif str(tree.func) == "max":
				return parseFormula("max(" + formulaToString(self.translateForSedml(tree.args[0], level, version)) + ")")
			elif str(tree.func) == "sum":
				return parseFormula("sum(" + formulaToString(self.translateForSedml(tree.args[0], level, version)) + ")")
			elif str(tree.func) == "product":
				return parseFormula("product(" + formulaToString(self.translateForSedml(tree.args[0], level, version)) + ")")
			else:
				raise SedmlMathException("SedmlMathWriter : Unknown Sympy Function %s" % str(tree.func))
		else:
			raise SedmlMathException("SedmlMathWriter : Unknown Sympy Symbol %s" % str(tree))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.numl.DimensionDescription import DimensionDescription
from libsignetsim.numl.AtomicDescription import AtomicDescription
from libsignetsim.settings.Settings import Settings

class TupleDescription (DimensionDescription):

	def __init__(self, document, name=None):
		DimensionDescription.__init__(self, document, name)
		self.__document = document
		self.__atomicDescriptions = []

	def createAtomicDescription(self, name=None, value_type="double"):
		atomic_description = AtomicDescription(self.__document, name, value_type)
		atomic_description.setId("%s_0" % self.getId())
		self.__atomicDescriptions.append(atomic_description)
		return atomic_description

	def readNuML(self, tuple_description, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		DimensionDescription.readNuML(self, tuple_description, level, version)

		for i in range(tuple_description.size()):
			t_atomic_description = AtomicDescription(self.__document)
			t_atomic_description.readNuML(tuple_description.getAtomicDescription(i), level, version)
			self.__atomicDescriptions.append(t_atomic_description)

	def writeNuML(self, tuple_description, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		DimensionDescription.writeNuML(self, tuple_description, level, version)
		for atomic_description in self.__atomicDescriptions:
			t_atomic_description = tuple_description.createAtomicDescription()
			atomic_description.writeNuML(t_atomic_description, level, version)

	def getAtomicDescriptions(self):
		return self.__atomicDescriptions


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.sbml.SbmlObject import SbmlObject

from libsignetsim.model.sbml.SpeciesReference import SpeciesReference
from libsignetsim.settings.Settings import Settings


class ListOfSpeciesReference(ListOf, HasIds, SbmlObject):
	""" Handles the list of species reference of a sbml model """

	def __init__ (self, model):

		self.__model = model
		ListOf.__init__(self, model)
		HasIds.__init__(self, model)
		SbmlObject.__init__(self, model)


	def readSbml(self, sbmlListOfSpeciesReference,
					sbmlLevel=Settings.defaultSbmlLevel,
					sbmlVersion=Settings.defaultSbmlVersion):

		""" Read the list of species references from a sbml model """

		for sbmlSpeciesReference in sbmlListOfSpeciesReference:
			speciesReference = SpeciesReference(self.__model, self.nextId())
			speciesReference.readSbml(sbmlSpeciesReference, sbmlLevel, sbmlVersion)
			ListOf.add(self, speciesReference)

		SbmlObject.readSbml(self, sbmlListOfSpeciesReference, sbmlLevel, sbmlVersion)


	def writeSbml(self, sbmlReaction,
					sbmlLevel=Settings.defaultSbmlLevel,
					sbmlVersion=Settings.defaultSbmlVersion):

		""" Write the list of species references to a sbml model """

		for speciesReference in self:
			speciesReference.writeSbml(sbmlReaction, sbmlLevel, sbmlVersion)

		if len(self) > 0:
			SbmlObject.writeSbml(self, sbmlReaction, sbmlLevel, sbmlVersion)


	def new(self):
		""" Add a new species reference to the list and returns it """

		t_speciesReference = SpeciesReference(self.__model, self.nextId())
		ListOf.add(self, t_speciesReference)
		return t_speciesReference


	def add(self, species, stoichiometry=1):
		t_sr = self.new()
		t_sr.setSpecies(species)
		t_sr.setStoichiometry(stoichiometry)


	def copy(self, obj, deletions=[], sids_subs={}, symbols_subs={}):

		if obj not in deletions:
			SbmlObject.copy(self, obj)
			for speciesReference in obj:
				if speciesReference not in deletions:
					t_sr = SpeciesReference(self.__model, self.nextId())
					t_sr.copy(speciesReference, sids_subs=sids_subs, symbols_subs=symbols_subs)
					ListOf.add(self, t_sr)

	def hasVariableStoichiometry(self):

		for sr in self:
			if sr.isVariableStoichiometry():
				return True
		return False


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.settings.Settings import Settings
from libsignetsim.numl.NMBase import NMBase
from libsignetsim.numl.CompositeDescription import CompositeDescription
# from libsignetsim.numl.AtomicDescription import AtomicDescription
# from libsignetsim.numl.TupleDescription import TupleDescription
from libsignetsim.numl.CompositeValue import CompositeValue
# from libsignetsim.numl.TupleValue import TupleValue
# from libsignetsim.numl.AtomicValue import AtomicValue
from re import sub

class ResultComponent (NMBase):

	def __init__(self, document):

		NMBase.__init__(self, document)
		self.__document = document
		self.__id = None
		self.__dimensionDescriptions = []
		self.__dimensions = []

	def createCompositeDescription(self, name=None, index_type="string"):

		dimension_description = CompositeDescription(self.__document, name, index_type)
		dimension_description.setId("%s_header_%d" % (self.__id, len(self.__dimensionDescriptions)))
		self.__dimensionDescriptions.append(dimension_description)
		return dimension_description

	# def createTupleDescription(self, name=None):
	#
	# 	dimension_description = TupleDescription(self.__document, name)
	# 	dimension_description.setId("%s_header_%d" % (self.__id, len(self.__dimensionDescriptions)))
	# 	self.__dimensionDescriptions.append(dimension_description)
	# 	return dimension_description

	def createCompositeValue(self, description, index_value="string"):

		dimension_value = CompositeValue(self.__document, self, description, index_value)
		self.__dimensions.append(dimension_value)
		return dimension_value

	# def createTupleValue(self, description):
	#
	# 	dimension_value = TupleValue(self.__document, self, description)
	# 	self.__dimensions.append(dimension_value)
	# 	return dimension_value

	def readNuML(self, result_component, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):
		NMBase.readNuML(self, result_component, level, version)
		self.__id = result_component.getId()

		# Always one element ??!! A composite one ??!!
		for dimension_description in result_component.getDimensionDescription():

			t_dimension_description = None
			# if dimension_description.isContentCompositeDescription():
			t_dimension_description = CompositeDescription(self.__document)
			#
			# elif dimension_description.isContentTupleDescription():
			# 	t_dimension_description = TupleDescription(self.__document)
			#
			# elif dimension_description.isContentAtomicDescription():
			# 	t_dimension_description = AtomicDescription(self.__document)

			if t_dimension_description is not None:
				t_dimension_description.readNuML(dimension_description, level, version)
				self.__dimensionDescriptions.append(t_dimension_description)

		for dimension in result_component.getDimension():

			t_dimension = None
			# if dimension.isContentCompositeValue():
			t_dimension = CompositeValue(self.__document, self, self.__dimensionDescriptions[0])

			# elif dimension.isContentTuple():
			# 	t_dimension = TupleValue(self.__document, self, self.__dimensionDescriptions[0])
			#
			# elif dimension.isContentAtomicValue():
			# 	t_dimension = AtomicValue(self.__document, self.__dimensionDescriptions[0])

			if t_dimension is not None:
				t_dimension.readNuML(dimension, level, version)
				self.__dimensions.append(t_dimension)

	def writeNuML(self, result_component, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):

		NMBase.writeNuML(self, result_component, level, version)

		result_component.setId(self.__id)

		for dimension_description in self.__dimensionDescriptions:
			t_dimension_description = None
			if isinstance(dimension_description, CompositeDescription):
				t_dimension_description = result_component.createCompositeDescription()

			# elif isinstance(dimension_description, TupleDescription):
			# 	t_dimension_description = result_component.createTupleDescription()
			#
			# elif isinstance(dimension_description, AtomicDescription):
			# 	t_dimension_description = result_component.createAtomicDescription()

			if t_dimension_description is not None:
				dimension_description.writeNuML(t_dimension_description, level, version)

		for dimension in self.__dimensions:

			t_dimension = None
			if isinstance(dimension, CompositeValue):
				t_dimension = result_component.createCompositeValue()

			# elif isinstance(dimension, TupleValue):
			# 	t_dimension = result_component.createTupleValue()
			#
			# elif isinstance(dimension, AtomicValue):
			# 	t_dimension = result_component.createAtomicValue()

			if t_dimension is not None:
				dimension.writeNuML(t_dimension, level, version)


	def setId(self, result_component_id):
		self.__id = result_component_id

	def getDimensionsDescriptions(self):
		return self.__dimensionDescriptions

	def getDimensions(self):
		return self.__dimensions


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf
from libsignetsim.sedml.Task import Task
from libsignetsim.sedml.RepeatedTask import RepeatedTask
from libsignetsim.settings.Settings import Settings

import libsbml
from libsedml import SEDML_TASK, SEDML_TASK_REPEATEDTASK
from six.moves import reload_module
reload_module(libsbml)


class ListOfTasks(ListOf):

	def __init__(self, document):

		ListOf.__init__(self, document)

		self.__document = document
		self.__taskCounter = 0

	def new(self, task_type, task_id=None):

		t_task_id = task_id
		if t_task_id is None:
			t_task_id = "task_%d" % self.__taskCounter

		if task_type == SEDML_TASK:
			task = Task(self.__document)
			task.setId(t_task_id)
			ListOf.append(self, task)
			self.__taskCounter += 1
			return task

		elif task_type == SEDML_TASK_REPEATEDTASK:
			task = RepeatedTask(self.__document)
			task.setId(t_task_id)
			ListOf.append(self, task)
			self.__taskCounter += 1
			return task

	def createTask(self, task_id=None):
		return self.new(SEDML_TASK, task_id)

	def createRepeatedTask(self, task_id=None):
		return self.new(SEDML_TASK_REPEATEDTASK, task_id)

	def readSedml(self, list_of_tasks, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_tasks, level, version)

		for t_task in list_of_tasks:

			if t_task.getTypeCode() == SEDML_TASK:
				task = Task(self.__document)
				task.readSedml(t_task, level, version)
				ListOf.append(self, task)
				self.__taskCounter += 1

			elif t_task.getTypeCode() == SEDML_TASK_REPEATEDTASK:
				task = RepeatedTask(self.__document)
				task.readSedml(t_task, level, version)
				ListOf.append(self, task)
				self.__taskCounter += 1

	def writeSedml(self, list_of_tasks, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_tasks, level, version)

		for t_task in self:

			if isinstance(t_task, Task):
				task = list_of_tasks.createTask()
				t_task.writeSedml(task, level, version)

			elif isinstance(t_task, RepeatedTask):
				task = list_of_tasks.createRepeatedTask()
				t_task.writeSedml(task, level, version)

	def getTask(self, task_reference):

		for task in self:
			if task.getId() == task_reference:
				return task

	def buildTasks(self):

		for task in self:
			task.build()

	def runTasks(self):

		for task in self:
			task.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function

from builtins import str
from builtins import object
from libsignetsim import SedmlDocument, Settings

from os.path import join, exists, isdir
from os import mkdir

class TestSuiteCase(object):

	def __init__ (self, case_id, sbml_level, sbml_version, test_export=True):

		self.caseId = "%05d" % case_id
		self.sbmlLevel = sbml_level
		self.sbmlVersion = sbml_version
		self.testExport = test_export

		self.document = None
		self.sbmlIdToCheck = None
		self.loadSBMLTestSuiteSettings()
		self.loadTestCaseModel()

	def run(self):

		self.document.run()
		return self.checkResults()

	def runTestSuiteWraper(self):

		self.document.run()
		self.writeSbmlTestOutput()

	def getFilename(self):

		return join(Settings.sbmlTestCasesPath, "%s/%s-sbml-l%sv%s-sedml.xml" % (
				self.caseId, self.caseId, self.sbmlLevel, self.sbmlVersion)
		)

	def getResultsFilename(self):
		return join(Settings.sbmlTestResultsPath, "%s.csv" % self.caseId)

	def getExpectedResultsFilename(self):
		return join(Settings.sbmlTestCasesPath, "%s/%s-results.csv" % (
				self.caseId, self.caseId)
		)

	def getSettingsFilename(self):

		return join(Settings.sbmlTestCasesPath, "%s/%s-settings.txt" % (
				self.caseId, self.caseId)
		)

	def getTemporarySedmlFilename(self):
		return join(Settings.sbmlTestResultsPath, "%s/%s-sbml-l%sv%s-sedml.xml" % (self.caseId, self.caseId, self.sbmlLevel, self.sbmlVersion))

	def getTemporaryPath(self):
		return join(Settings.sbmlTestResultsPath, "%s" % self.caseId)

	def loadSBMLTestSuiteSettings(self):

		self.sbmlIdToCheck = []
		settings = open(self.getSettingsFilename(), 'r')

		for line in settings:

			if line.startswith("variables:"):
				res_split = line.split(":", 2)
				if len(res_split[1].strip()) > 0:
					res_split = res_split[1].strip().split(",")
					for t_var in res_split:
						self.sbmlIdToCheck.append(t_var.strip())

			if line.startswith("absolute:"):
				res_split = line.split(":", 2)
				self.testAbsTol = float(res_split[1].strip())

			if line.startswith("relative:"):
				res_split = line.split(":", 2)
				self.testRelTol = float(res_split[1].strip())

		settings.close()


	def loadTestCaseModel(self):

		self.document = SedmlDocument()
		self.document.readSedmlFromFile(self.getFilename())

		if self.testExport:

			if not isdir(self.getTemporaryPath()):
				mkdir(self.getTemporaryPath())

			self.document.writeSedmlToFile(
				self.getTemporarySedmlFilename(),
				write_sbml_dependencies=True
			)

			self.document = SedmlDocument()
			self.document.readSedmlFromFile(self.getTemporarySedmlFilename())

	def checkResults(self):

		expected_results = open(self.getExpectedResultsFilename())

		variables = []
		timepoints = []
		for i, line in enumerate(expected_results.readlines()):

			if i == 0:
				t_variables = line.strip().split(',')
				if t_variables[len(t_variables)-1] == '':
					t_variables = t_variables[:-1]
				variables = [t_variable.strip() for t_variable in t_variables]
			else:
				t_timepoints = line.strip().split(',')
				if t_timepoints[len(t_timepoints)-1] == '':
					t_timepoints = t_timepoints[:-1]
				timepoints.append([float(t_timepoint) for t_timepoint in t_timepoints])

		expected_results.close()

		simulated_results = self.document.listOfOutputs.getReports()[0].getData()

		result = True
		precisionError = False
		precisionErrorVariables = []

		for i_timepoint, timepoint in enumerate(timepoints):

			for i_var, var in enumerate(variables):

				t_expected_value = float(timepoint[i_var])

				if var == 'time':
					pass

				else:
					t_value = simulated_results[var][i_timepoint]

					if abs(t_value-t_expected_value) > (self.testAbsTol + self.testRelTol*abs(t_expected_value)):
						result = False
						precisionError = True
						if var not in precisionErrorVariables:
							precisionErrorVariables.append(var)
						# print "%.10g, %.10g (%10g, %.2g)" % (t_value, t_expected_value, abs(t_value-t_expected_value), simulated_results['Time'][i_timepoint])



		if precisionError:
			print("> Precision error in variables : %s" % str(precisionErrorVariables))
			return False

		return result

	def writeSbmlTestOutput(self):

		data = self.document.listOfOutputs.getReports()[0].getData()

		if not exists(Settings.sbmlTestResultsPath):
			mkdir(Settings.sbmlTestResultsPath)

		results_file = open(self.getResultsFilename(), 'w')
		# (traj_times, trajs) = self.rawData[0]

		# Writing header
		line = "time"

		for sbml_id in self.sbmlIdToCheck:
			line += ",%s" % sbml_id

		results_file.write(line + "\n")

		for i_t, t_time in enumerate(data['Time']):
			line = "%.14f" % t_time

			for sbml_id in self.sbmlIdToCheck:
				line += ",%.14f" % data[sbml_id][i_t]

			results_file.write(line + "\n")

		results_file.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.math.sympy_shortcuts import SympySymbol, SympyInteger
from sympy import simplify, diff, solve


class MathJacobianMatrix(object):


	""" Sbml model class """

	def __init__ (self):
		""" Constructor of model class """

		self.interactionMatrix = None
		self.jacobianMatrix = None


	def buildJacobianMatrix(self, including_fast_reactions=True):

		self.interactionMatrix = []
		self.jacobianMatrix = []

		for ode in self.ODEs:

			partial_ders = []
			signs = []

			for var in self.ODE_vars:

				t_part_der = diff(ode.getDeveloppedInternalMathFormula(), var.getDeveloppedInternalMathFormula())
				t_part_der_formula = MathFormula(self)
				t_part_der_formula.setInternalMathFormula(t_part_der)
				partial_ders.append(t_part_der_formula)

				if t_part_der == MathFormula.ZERO:
					signs.append(0)

				else:
					for atom in t_part_der.atoms(SympySymbol):
						t_part_der = t_part_der.subs({atom: SympyInteger(1)})

					if t_part_der > MathFormula.ZERO:
						signs.append(1)
					elif t_part_der < MathFormula.ZERO:
						signs.append(-1)
					else:
						signs.append(2)

			self.jacobianMatrix.append(partial_ders)
			self.interactionMatrix.append(signs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
# from __future__ import print_function

from libsignetsim.settings.Settings import Settings
from libsignetsim.optimization.OptimizationException import OptimizationCompilationException, OptimizationExecutionException
from time import time, sleep
from os.path import join, getsize, isfile
from os import getcwd, setpgrp, mkdir
from subprocess import call
from math import log10
from threading import Thread


class OptimizationExecution(object):


	OPTIM_FAILURE       =	-1
	OPTIM_SUCCESS       =	0
	OPTIM_RUNNING		=	1
	OPTIM_INTERRUPTED	= 	2
	OPTIM_INITIALIZED 	= 	3


	def __init__ (self):

		self.startTime = None
		self.stopTime = None
		self.elapsedTime = None
		self.monitor = False
		self.optimizationId = int(time()*1000)
		self.directory = None
		self.finalScore = None
		self.status = self.OPTIM_INITIALIZED
		# mkdir(self.getTempDirectory())


	def getTempDirectory(self):

		if self.directory is None:
			return join(Settings.tempDirectory,	"optimization_%s/" % str(self.optimizationId))

		else:
			return join(self.directory, "optimization_%s/" % str(self.optimizationId))

	def setTempDirectory(self, directory):

		self.directory = directory



	def compile(self, nb_procs):

		mkdir(join(self.getTempDirectory(), "logs"))
		mkdir(join(self.getTempDirectory(), "logs", "score"))
		mkdir(join(self.getTempDirectory(), "logs", "res"))

		if nb_procs > 1:
			target = "lsa.mpi"

		else:
			target = "lsa"

		cmd_comp = "make -f %sMakefile -C %s %s 1>/dev/null" % (
								self.getTempDirectory(),
								self.getTempDirectory(),
								target)

		with open(join(self.getTempDirectory(), "out_optim_comp"), "w") as stdout,  open(join(self.getTempDirectory(), "err_optim_comp"), "w") as stderr:
			res_comp = call(cmd_comp, stdout=stdout, stderr=stderr, shell=True, preexec_fn=setpgrp, close_fds=True)

		timeout = 3
		i = 0
		while not isfile(join(self.getTempDirectory(), "err_optim_comp")) or i == timeout:
			sleep(1)
			i += 1

		if res_comp != 0 or getsize(join(self.getTempDirectory(), "err_optim_comp")) > 0:
			self.status = self.OPTIM_FAILURE
			if Settings.verbose >= 1:
				print("-"*40 + "\n")
				print("> Error during optimization compilation :")
				with open(join(self.getTempDirectory(), "err_optim_comp"), 'r') as f_err_comp:
					for line in f_err_comp:
						print(line)

				print("-"*40 + "\n")
			raise OptimizationCompilationException("Error during optimization compilation")
		else:
			self.status = self.OPTIM_SUCCESS
			return self.OPTIM_SUCCESS


	def run(self, nb_procs, timeout, maxiter):

		t_timeout = ""
		if timeout is not None and timeout > 0:
			t_timeout = "-d %d " % timeout

		s_maxiter = ""
		if maxiter is not None and maxiter > 0:
			s_maxiter = "-m %d " % maxiter

		present_dir = getcwd()

		if nb_procs > 1:
			target = "cd %s; mpirun -np %d ./lsa.mpi; cd %s" % (self.getTempDirectory(), nb_procs, present_dir)

		else:
			target = "cd %s; ./lsa; cd %s" % (self.getTempDirectory(), present_dir)

		t_command_line = "%s" % target

		self.status = self.OPTIM_RUNNING
		with open(join(self.getTempDirectory(), "out_optim"), "w") as stdout, open(join(self.getTempDirectory(), "err_optim"), "w") as stderr:
			res_optim = call(t_command_line,
					stdout=stdout, stderr=stderr,
					shell=True, preexec_fn=setpgrp, close_fds=True)

		if res_optim != 0 and res_optim != 124:

			self.stopTime = int(time())
			self.elapsedTime = self.stopTime - self.startTime
			self.status = self.OPTIM_FAILURE
			raise OptimizationExecutionException("Optim execution returned %s" % res_optim)

		timeout = 3
		i = 0
		while not isfile(join(self.getTempDirectory(), "err_optim")) or i == timeout:
			sleep(1)
			i += 1

		if getsize(join(self.getTempDirectory(), "err_optim")) > 0:

			# There is some weird error here, apparently caused by docker filesystem.
			# ** apparently ** we can ignore it
			non_docker_err = False
			with open(self.getTempDirectory() + "err_optim", 'r') as f_err_optim:
				for line in f_err_optim:
					if not line.startswith("Unexpected end of /proc/mounts"):
						non_docker_err = True


			if non_docker_err:
				if Settings.verbose >= 1:
					print("-" * 40 + "\n")
					print("> Error during optimization execution :")
					with open(join(self.getTempDirectory(), "err_optim"), 'r') as f_err:
						for line in f_err:
							print(line)

					print("-" * 40 + "\n")

				err = open(join(self.getTempDirectory(), "err_optim"))

				if err.readline() != "mpirun: killing job...\n":
					err.close()
					self.stopTime = int(time())
					self.elapsedTime = self.stopTime - self.startTime
					self.status = self.OPTIM_FAILURE
					raise OptimizationExecutionException("Error during optimization execution")

				err.close()

		if isfile(join(self.getTempDirectory(), "pid")):
			self.status = self.OPTIM_INTERRUPTED
		else:
			self.status = self.OPTIM_SUCCESS

		return self.OPTIM_SUCCESS


	def run_inside_thread(self, success, failure, nb_procs=2, timeout=None, maxiter=None):

		try:
			res = self.runOptimization(nb_procs=nb_procs, timeout=timeout, maxiter=maxiter)

			if res != self.OPTIM_FAILURE:
				if success is not None:
					success(self)
			else:
				if failure is not None:
					failure(self)

		except Exception as e:
			if failure is not None:
				failure(self, e)


	def run_async(self, success, failure, nb_procs=2, timeout=None, maxiter=None):

		t = Thread(group=None, target=self.run_inside_thread, args=(success, failure, nb_procs, timeout, maxiter))
		t.setDaemon(True)
		t.start()
		return t

	def restart_inside_thread(self, success, failure, nb_procs=2, timeout=None, maxiter=None):

		try:
			res = self.restartOptimization(nb_procs=nb_procs, timeout=timeout, maxiter=maxiter)

			if res != self.OPTIM_FAILURE:
				if success is not None:
					success(self)
			else:
				if failure is not None:
					failure(self)

		except Exception as e:
			if failure is not None:
				failure(self, e)


	def restart_async(self, success, failure, nb_procs=2, timeout=None, maxiter=None):

		t = Thread(group=None, target=self.restart_inside_thread, args=(success, failure, nb_procs, timeout, maxiter))
		t.setDaemon(True)
		t.start()
		return t

	def readFinalScore(self):

		file_final_score = open(join(self.getTempDirectory(), "logs", "score", "score"))
		final_score = float(file_final_score.readline())

		final_score = max(round(final_score, int(-log10(Settings.defaultPlsaCriterion))),
							Settings.defaultPlsaCriterion)

		file_final_score.close()


		if Settings.verbose >= 1:
			print("> Optimization executed. Final score : %.5g" % final_score)
		return final_score

	def restartOptimization(self, nb_procs=2, timeout=None, maxiter=None):

		self.startTime = int(time())

		res_2 = self.run(nb_procs, timeout, maxiter)
		if res_2 == self.OPTIM_SUCCESS:

			self.stopTime = int(time())
			self.elapsedTime = self.stopTime - self.startTime
			self.finalScore = self.readFinalScore()
			return self.finalScore
		elif Settings.verbose >= 1:
			print("> Execution failed !")

		self.stopTime = int(time())
		self.elapsedTime = self.stopTime - self.startTime

		return self.OPTIM_FAILURE


	def runOptimization(self, nb_procs, timeout=None, maxiter=None):

		# if self.monitor:
		# t = OptimizationMonitor(self)  #threading.Thread(target=self.monitorOptimization)
		# t.start()

		self.startTime = int(time())

		res = self.compile(nb_procs)

		if res == self.OPTIM_SUCCESS:

			res_2 = self.run(nb_procs, timeout, maxiter)
			if res_2 == self.OPTIM_SUCCESS:

				self.stopTime = int(time())
				self.elapsedTime = self.stopTime - self.startTime
				self.finalScore = self.readFinalScore()
				return self.finalScore
			elif Settings.verbose >= 1:
				print("> Execution failed !")
		elif Settings.verbose >= 1:
			print("> Compilation failed !")

		self.stopTime = int(time())
		self.elapsedTime = self.stopTime - self.startTime

		return self.OPTIM_FAILURE

	def isInterrupted(self):
		return self.status == self.OPTIM_INTERRUPTED
	def hasSucceeded(self):
		return self.status == self.OPTIM_SUCCESS
	def hasFailed(self):
		return self.status == self.OPTIM_FAILURE
	def hasStarted(self):
		return self.status != self.OPTIM_INITIALIZED


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function



from libsignetsim.settings.Settings import Settings
from time import time
from sympy import Symbol
from libsignetsim.model.math.MathFormula import MathFormula


class CModelWriter(object):
	""" Sbml model class """

	def __init__ (self, obj_id=0):
		""" Constructor of model class """

		self.objId = obj_id

	def writeCCode(self, f_h, f_c, i_model, time_min, list_samples, abs_tol, rel_tol):

		t0 = time()
		self.writeSimulationInitialization(f_h, f_c, i_model, time_min, list_samples, abs_tol, rel_tol)
		if Settings.verboseTiming >= 2:
			print(">>> Initialization written in %.2gs" % (time()-t0))

		self.writeSimulationFinalization(f_h, f_c, i_model)
		self.writeInitialAssignments(f_h, f_c, i_model)
		t0 = time()
		if len(self.getMathModel().listOfDAEs) > 0:
			self.writeIdaSimulationFunction(f_h, f_c, i_model)
		else:
			self.writeCVodeSimulationFunction(f_h, f_c, i_model)
		if Settings.verboseTiming >= 2:
			print(">>> System written in %.2gs" % (time() - t0))

		t0 = time()
		self.writeSimulationComputeRules(f_h, f_c, i_model)
		if Settings.verboseTiming >= 2:
			print(">>> Rules written in %.2gs" % (time() - t0))

		t0 = time()
		self.writeEventsTriggersFunction(f_h, f_c, i_model)
		self.writeEventsActivationFunction(f_h, f_c, i_model)
		self.writeEventsAssignmentFunction(f_h, f_c, i_model)
		self.writeEventsPriorityFunction(f_h, f_c, i_model)
		if Settings.verboseTiming >= 2:
			print(">>> Events written in %.2gs" % (time() - t0))

	def writeSimulationInitialization(self, f_h, f_c, model_id, time_min, list_samples, abs_tol, rel_tol):
		""" Writes the model initialization function in C files """

		variable_name = "model_%d" % model_id

		f_h.write("void init_model_%d();\n" % model_id)
		f_c.write("void init_model_%d()\n{\n" % model_id)

		f_c.write("  %s.nb_derivative_variables = %d;\n" % (variable_name, self.getMathModel().nbOdes))
		f_c.write("  %s.nb_assignment_variables = %d;\n" % (variable_name, self.getMathModel().nbAssignments))
		f_c.write("  %s.nb_constant_variables = %d;\n" % (variable_name, self.getMathModel().nbConstants))
		f_c.write("  %s.nb_algebraic_variables = %d;\n" % (variable_name, self.getMathModel().nbAlgebraics))

		if self.getMathModel().nbOdes > 0:
			f_c.write("  %s.derivative_variables = (ModelVariable *) malloc(sizeof(ModelVariable) * %s.nb_derivative_variables);\n" % (variable_name, variable_name))

		if self.getMathModel().nbAssignments > 0:
			f_c.write("  %s.assignment_variables = (ModelVariable *) malloc(sizeof(ModelVariable) * %s.nb_assignment_variables);\n" % (variable_name, variable_name))

		if self.getMathModel().nbConstants > 0:
			f_c.write("  %s.constant_variables = (ModelVariable *) malloc(sizeof(ModelVariable) * %s.nb_constant_variables);\n" % (variable_name, variable_name))

		if self.getMathModel().nbAlgebraics > 0:
			f_c.write("  %s.algebraic_variables = (ModelVariable *) malloc(sizeof(ModelVariable) * %s.nb_algebraic_variables);\n" % (variable_name, variable_name))

		if len(self.getMathModel().listOfDAEs) > 0:
			if self.getMathModel().nbOdes > 0:
				f_c.write("  %s.der_der_variables = (ModelVariable *) malloc(sizeof(ModelVariable) * %s.nb_derivative_variables);\n" % (variable_name, variable_name))
			if self.getMathModel().nbAlgebraics > 0:
				f_c.write("  %s.alg_der_variables = (ModelVariable *) malloc(sizeof(ModelVariable) * %s.nb_algebraic_variables);\n" % (variable_name, variable_name))

		for i_var, variable_ode in enumerate(self.getMathModel().variablesOdes):
			t_value = self.getMathModel().listOfInitialConditions[variable_ode.symbol.getSymbol()]

			f_c.write("  %s.derivative_variables[%d] = (ModelVariable) {%s, \"%s\", VAR_DERIVATIVE};\n" % (
							variable_name, i_var, t_value.getCMathFormula(), variable_ode.getSymbolStr()))

			if len(self.getMathModel().listOfDAEs) > 0:
				f_c.write("  %s.der_der_variables[%d] = (ModelVariable) {RCONST(0.0), \"%s\", VAR_DER_DER};\n" % (
								variable_name, i_var, variable_ode.symbol.getPrettyPrintMathFormula()))

		for i_var, variable_ass in enumerate(self.getMathModel().variablesAssignment):
			t_value = self.getMathModel().listOfInitialConditions[variable_ass.symbol.getSymbol()]
			f_c.write("  %s.assignment_variables[%d] = (ModelVariable) {%s, \"%s\", VAR_ASSIGNMENT};\n" % (
								variable_name, i_var, t_value.getCMathFormula(), variable_ass.getSymbolStr()))

		for i_var, variable_cst in enumerate(self.getMathModel().variablesConstant):
			t_value = self.getMathModel().listOfInitialConditions[variable_cst.symbol.getSymbol()]
			f_c.write("  %s.constant_variables[%d] = (ModelVariable) {%s, \"%s\", VAR_CONSTANT};\n" % (
								variable_name, i_var, t_value.getCMathFormula(), variable_cst.getSymbolStr()))


		for i_var, variable_alg in enumerate(self.getMathModel().variablesAlgebraic):
			t_value = self.getMathModel().listOfInitialConditions[variable_alg.symbol.getSymbol()]
			f_c.write("  %s.algebraic_variables[%d] = (ModelVariable) {%s, \"%s\", VAR_ALGEBRAIC};\n" % (
								variable_name, i_var, t_value.getCMathFormula(), variable_alg.getSymbolStr()))


			if len(self.getMathModel().listOfDAEs) > 0:
				f_c.write("  %s.alg_der_variables[%d] = (ModelVariable) {RCONST(0.0), \"%s\", VAR_ALG_DER};\n" % (
								variable_name, i_var, variable_alg.getSymbolStr()))

		f_c.write("  %s.nb_init_assignments = %d;\n" % (variable_name, len(self.getMathModel().listOfInitialAssignments)))
		f_c.write("  %s.nb_events = %d;\n" % (variable_name, self.getMathModel().listOfEvents.nbValidEvents()))
		f_c.write("  %s.nb_roots = %d;\n" % (variable_name, self.getMathModel().listOfEvents.nbRoots()))

		if self.getMathModel().listOfEvents.nbRoots() > 0:
			f_c.write("  %s.roots_operators = calloc(%d, sizeof(int));\n" % (variable_name, len(self.getMathModel().listOfEvents.getRootsOperators())))
			for i, roots_operators in enumerate(self.listOfEvents.getRootsOperators()):
				f_c.write("  %s.roots_operators[%d] = %d;\n" % (variable_name, i, roots_operators))

		if self.getMathModel().listOfEvents.nbValidEvents() > 0:
			f_c.write("  %s.events_init = calloc(%s.nb_events, sizeof(int));\n" % (variable_name, variable_name))
			for i, event in enumerate(self.getMathModel().listOfEvents.validEvents()):
				f_c.write("  %s.events_init[%d] = %d;\n" % (variable_name, i, event.trigger.initialValue))

			f_c.write("  %s.memory_size_per_event = calloc(%s.nb_events, sizeof(int));\n" % (variable_name, variable_name))
			for event in self.getMathModel().listOfEvents.validEvents():
				f_c.write("  %s.memory_size_per_event[%d] = %d;\n" % (variable_name, event.objId, event.memorySize()))

			f_c.write("  %s.events_has_priority = calloc(%s.nb_events, sizeof(int));\n" % (variable_name, variable_name))
			for event in self.getMathModel().listOfEvents.validEvents():
				f_c.write("  %s.events_has_priority[%d] = %d;\n" % (variable_name, event.objId, event.priority is not None))

		list_samples_str = "%.16g" % list_samples[0]

		for i in range(1, len(list_samples)):
			list_samples_str += ", %.16g" % list_samples[i]

		f_c.write("  %s.integration_settings = malloc(sizeof(IntegrationSettings));\n" % variable_name)
		f_c.write("  %s.integration_settings->t_min = %g;\n" % (variable_name, time_min))
		f_c.write("  %s.integration_settings->nb_samples = %d;\n" % (variable_name, len(list_samples)))
		f_c.write("  %s.integration_settings->abs_tol = %g;\n" % (variable_name, abs_tol))
		f_c.write("  %s.integration_settings->rel_tol = %g;\n" % (variable_name, rel_tol))
		f_c.write("  static double t_list_samples[%d] = {%s};\n" % (len(list_samples), list_samples_str))
		f_c.write("  %s.integration_settings->list_samples = t_list_samples;\n" % (variable_name))

		f_c.write("  %s.integration_functions = malloc(sizeof(IntegrationFunctions));\n" % variable_name)
		f_c.write("  %s.integration_functions->initAssPtr = &init_assignments_%d;\n" % (variable_name, model_id))
		if len(self.getMathModel().listOfDAEs) > 0:
			f_c.write("  %s.integration_functions->funcIdaPtr = &func_ida_%d;\n" % (variable_name, model_id))
			f_c.write("  %s.integration_functions->isDAE = 1;\n" % variable_name)
			f_c.write("  %s.integration_functions->init_conditions_solved = 0;\n" % (variable_name))
		else:
			f_c.write("  %s.integration_functions->isDAE = 0;\n" % variable_name)
			f_c.write("  %s.integration_functions->funcPtr = &func_cvode_%d;\n" % (variable_name, model_id))

		f_c.write("  %s.integration_functions->assPtr = &compute_rules_%d;\n" % (variable_name, model_id))
		f_c.write("  %s.integration_functions->hasJacobian = 0;\n" % variable_name)

		if len(self.getMathModel().listOfDAEs) > 0:
			f_c.write("  %s.integration_functions->rootsEventsIDAPtr = &roots_events_%d;\n" % (variable_name, model_id))
		else:
			f_c.write("  %s.integration_functions->rootsEventsPtr = &roots_events_%d;\n" % (variable_name, model_id))
		f_c.write("  %s.integration_functions->activateEventsPtr = &activate_events_%d;\n" % (variable_name, model_id))
		f_c.write("  %s.integration_functions->assignEventsPtr = &assign_events_%d;\n" % (variable_name, model_id))
		f_c.write("  %s.integration_functions->priorityEventsPtr = &priority_events_%d;\n" % (variable_name, model_id))

		f_c.write("  %s.integration_options = malloc(sizeof(IntegrationOptions));\n" % variable_name)
		f_c.write("  %s.integration_options->max_num_steps = %g;\n" % (variable_name, Settings.defaultCVODEmaxNumSteps))
		f_c.write("  %s.integration_options->max_conv_fails = %g;\n" % (variable_name, Settings.defaultCVODEMaxConvFails))
		f_c.write("  %s.integration_options->max_err_test_fails = %g;\n" % (variable_name, Settings.defaultCVODEMaxErrFails))
		f_c.write("  rt_set_precision(RCONST(%g), RCONST(%g));\n" % (abs_tol, rel_tol))
		f_c.write("\n}\n\n")

	def writeSimulationFinalization(self, f_h, f_c, model_id):
		""" Writes the model initialization function in C files """

		variable_name = "model_%d" % model_id

		f_h.write("void finalize_model_%d();\n" % model_id)
		f_c.write("void finalize_model_%d()\n{\n" % model_id)

		if self.getMathModel().nbOdes > 0:
			f_c.write("  free(%s.derivative_variables);\n" % variable_name)

		if self.getMathModel().nbAssignments > 0:
			f_c.write("  free(%s.assignment_variables);\n" % variable_name)

		if self.getMathModel().nbConstants > 0:
			f_c.write("  free(%s.constant_variables);\n" % variable_name)

		if len(self.getMathModel().listOfDAEs) > 0 and self.getMathModel().nbOdes > 0:
			f_c.write("  free(%s.der_der_variables);\n" % variable_name)


		if self.getMathModel().listOfEvents.nbRoots() > 0:
			f_c.write("  free(%s.roots_operators);\n" % variable_name)

		if len(self.getMathModel().listOfEvents) > 0:
			f_c.write("  free(%s.events_init);\n" % variable_name)
			f_c.write("  free(%s.memory_size_per_event);\n" % variable_name)
			f_c.write("  free(%s.events_has_priority);\n" % variable_name)

		f_c.write("  free(%s.integration_settings);\n" % variable_name)

		f_c.write("  free(%s.integration_functions);\n" % variable_name)

		f_c.write("  free(%s.integration_options);\n" % variable_name)

		f_c.write("}\n\n")



	def writeInitialAssignments(self, f_h, f_c, model_id):
		""" Writes the initial assignments in C files """

		f_h.write("int init_assignments_%d(realtype t, N_Vector y, void * user_data);\n" % model_id)
		f_c.write("int init_assignments_%d(realtype t, N_Vector y, void * user_data)\n" % model_id)
		f_c.write("{\n")
		f_c.write("  IntegrationData * data = (IntegrationData *) user_data;\n")
		f_c.write("  N_Vector cst = data->constant_variables;\n")
		f_c.write("  N_Vector ass = data->assignment_variables;\n")
		f_c.write("  compute_rules_%d(t, y, user_data);\n" % model_id)

		for t_init_assignment in self.getMathModel().listOfInitialAssignments:
			if t_init_assignment.isValid():
				f_c.write("  %s = %s;\n\n" % (
					t_init_assignment.getVariable().symbol.getCMathFormula(),
					t_init_assignment.getDefinition(rawFormula=True).getCMathFormula()
				))

		f_c.write("  return 0;\n")
		f_c.write("}\n\n")


	def writeIdaSimulationFunction(self, f_h, f_c, model_id):
		""" Writes the daes definition is C files """

		f_h.write("int func_ida_%d(realtype t, N_Vector y, N_Vector ydot, N_Vector r, void *user_data);\n" % model_id)
		f_c.write("int func_ida_%d(realtype t, N_Vector y, N_Vector ydot, N_Vector r, void *user_data)\n" % model_id)
		f_c.write("{\n")
		f_c.write("  IntegrationData * data = (IntegrationData *) user_data;\n")
		f_c.write("  N_Vector cst = data->constant_variables;\n")
		f_c.write("  N_Vector ass = data->assignment_variables;\n")
		f_c.write("  compute_rules_%d(t, y, user_data);\n" % model_id)

		i_var = 0
		for i_ode, t_ode in enumerate(self.getMathModel().listOfODEs):
			t_var = t_ode.getVariable()
			f_c.write("  // ODE\n")
			f_c.write("  Ith(r, %d) = %s - Ith(ydot, %d);\n\n" % (i_var+1, t_ode.getDefinition().getCMathFormula(), t_var.ind+1))
			i_var += 1

		for i_dae, t_dae in enumerate(self.getMathModel().listOfDAEs):
			f_c.write("  // DAE\n")
			f_c.write("  Ith(r, %d) = %s;\n\n" % (i_var+1, t_dae.getDefinition().getCMathFormula()))
			i_var += 1

		f_c.write("  return 0;\n")
		f_c.write("}\n\n")


	def writeCVodeSimulationFunction(self, f_h, f_c, model_id):
		""" Writes the odes definition in C files """

		f_h.write("int func_cvode_%d(realtype t, N_Vector y, N_Vector ydot, void *user_data);\n" % model_id)
		f_c.write("int func_cvode_%d(realtype t, N_Vector y, N_Vector ydot, void *user_data)\n" % model_id)
		f_c.write("{\n")

		if len(self.getMathModel().listOfODEs) > 0:

			f_c.write("  IntegrationData * data = (IntegrationData *) user_data;\n")
			f_c.write("  N_Vector cst = data->constant_variables;\n")
			f_c.write("  N_Vector ass = data->assignment_variables;\n")
			f_c.write("  compute_rules_%d(t,y, user_data);\n" % model_id)

			i_var = 0
			for i_ode, t_ode in enumerate(self.getMathModel().listOfODEs):
				t_var = t_ode.getVariable()
				t0 = time()
				t_c_ode = t_ode.getDefinition().getCMathFormula()
				if Settings.verboseTiming >= 2:
					print(">>> ODE #%s translated in %.2gs" % (i_ode, time() - t0))

				f_c.write("  // ODE\n")
				f_c.write("  Ith(ydot, %d) = %s;\n\n" % (t_var.ind+1, t_c_ode))
				i_var += 1

		f_c.write("  return 0;\n")
		f_c.write("}\n\n")



	def writeSimulationJacobianMatrixFunction(self, f_h, f_c, model_id):
		""" Writes the jacobian matrix definition in C files """

		f_h.write(("int jac_cvode_%d(long int N, realtype t, N_Vector y, N_Vector fy, "
				   "DlsMat J, void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);\n") % model_id)

		#Writing cvode_jacobian_matrix
		f_c.write(("int jac_cvode_%d(long int N, realtype t, N_Vector y, N_Vector fy, "
				   "DlsMat J, void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)\n") % model_id)
		f_c.write("{\n")

		# f_c.write("  IntegrationData * data = (IntegrationData *) user_data;\n")
		# f_c.write("  N_Vector cst = data->constant_variables;\n")
		# f_c.write("  N_Vector ass = data->assignment_variables;\n")
		# f_c.write("  compute_rules_%d(t,y, user_data);\n" % model_id)
		#
		# i_var = 0
		# for variable in self.listOfVariables.values():
		#     if variable.isRateRuled() or (not variable.constant and not variable.isRuled()):
		#
		#         i_var_der = 0
		#         for variable_der in self.listOfVariables.values():
		#             if variable_der.isRateRuled() or (not variable.constant and not variable.isRuled()):
		#
		#                 f_c.write("  IJth(J, %d, %d) = %s;\n\n" % (i_var+1, i_var_der+1, variable.getODEDerivative(MathFormula.MATH_C, variable_der)))
		#                 i_var_der += 1
		#
		#         i_var += 1

		f_c.write("  return 0;\n")
		f_c.write("}\n\n")


	def writeSimulationComputeRules(self, f_h, f_c, model_id):
		""" Writes the rules definition function in C files """

		f_h.write("int compute_rules_%d(realtype t, N_Vector y, void * user_data);\n" % model_id)
		f_c.write("int compute_rules_%d(realtype t, N_Vector y, void * user_data)\n{\n" % model_id)

		if len(self.getMathModel().listOfCFEs) > 0:
			f_c.write("  IntegrationData * data = (IntegrationData *) user_data;\n")
			f_c.write("  N_Vector cst = data->constant_variables;\n")
			f_c.write("  N_Vector ass = data->assignment_variables;\n")

			for i_cfe, t_cfe in enumerate(self.getMathModel().listOfCFEs):

				t_var = t_cfe.getVariable()
				f_c.write("  Ith(ass, %s) = %s;\n" % (
								t_var.ind+1,
								t_cfe.getDefinition().getCMathFormula()
				))

		f_c.write("  return 0;\n}\n")


	def writeEventsTriggersFunction(self, f_h, f_c, model_id):
		""" Writes the events conditions function in C files """

		if len(self.getMathModel().listOfDAEs) > 0:
			f_h.write("int roots_events_%d(realtype t, N_Vector y, N_Vector yp, realtype *gout,void *user_data);\n" % model_id)
			f_c.write("int roots_events_%d(realtype t, N_Vector y, N_Vector yp, realtype *gout,void *user_data)\n{\n" % model_id)
		else:
			f_h.write("int roots_events_%d(realtype t, N_Vector y, realtype *gout,void *user_data);\n" % model_id)
			f_c.write("int roots_events_%d(realtype t, N_Vector y, realtype *gout,void *user_data)\n{\n" % model_id)

		if self.getMathModel().listOfEvents.nbValidEvents() > 0:

			f_c.write("  IntegrationData * data = (IntegrationData *) user_data;\n")
			f_c.write("  N_Vector cst = data->constant_variables;\n")
			f_c.write("  N_Vector ass = data->assignment_variables;\n")
			f_c.write("  compute_rules_%d(t,y, user_data);\n" % model_id)

			i_event = 0


			for event in self.getMathModel().listOfEvents.validEvents():
				t_distances = event.trigger.getRootsFunctions()
				for t_distance in t_distances:
					f_c.write("  gout[%d] = %s;\n" % (i_event, t_distance))
					i_event += 1

		f_c.write("  return 0;\n}\n")



	def writeEventsActivationFunction(self, f_h, f_c, model_id):
		""" Writes the events conditions function in C files """

		f_h.write("int activate_events_%d(realtype t, N_Vector y, void * user_data);\n" % model_id)
		f_c.write("int activate_events_%d(realtype t, N_Vector y, void * user_data)\n{\n" % model_id)
		f_c.write("  IntegrationData * data = (IntegrationData *) user_data;\n")
		f_c.write("  N_Vector cst = data->constant_variables;\n")
		f_c.write("  N_Vector ass = data->assignment_variables;\n")

		i_roots = 0
		i_roots2 = 0

		for i_event, event in enumerate(self.getMathModel().listOfEvents.validEvents()):

			# Events deactivation
			(t_deactivation_condition, i_roots) = event.trigger.getDeactivationCondition(i_roots)

			f_c.write((
				"  if ((data->events_ready[%d] == 0) && %s)\n  {\n"
				+ "    data->events_ready[%d] = 1;\n") % (
					i_event, t_deactivation_condition, i_event
			))

			if not event.trigger.isPersistent:
				f_c.write("    data->events_triggers[%d]--;\n" % i_event)
				f_c.write("    untriggerChildren(data, %d);\n" % i_event)

			f_c.write("  }\n\n")

			# Events activation
			# Condition
			(t_activation_condition, i_roots2) = event.trigger.getActivationCondition(i_roots2)

			f_c.write((
				"  else if ((data->events_ready[%d] == 1) && %s)\n  {\n"
				+ "    data->events_ready[%d] = 0;\n") % (
					i_event, t_activation_condition, i_event
			))

			if event.delay is not None and event.delay.notZero():
				if not event.trigger.isPersistent:
					f_c.write("    retriggerChildren(data, %d);\n" % i_event)

				f_c.write("    realtype * memory = addTimedEvent(t+%s, %d, %d, user_data);\n" % (
					event.delay.getCMathFormula(), i_event, event.memorySize()
				))

			else:
				f_c.write("    data->events_triggers[%d]++;\n" % i_event)

			for i_assignment, event_assignment in enumerate(event.listOfEventAssignments):
				if event_assignment.isValid() and event.useValuesFromTriggerTime:
					if event.delay is not None and event.delay.notZero():
						f_c.write("    memory[%d] = %s;\n" % (
							i_assignment,
							event_assignment.getDefinition().getCMathFormula()
						))

					else:
						f_c.write("    data->events_memory[%d][%d] = %s;\n"	% (
							i_event, i_assignment,
							event_assignment.getDefinition().getCMathFormula()
						))

				else:
					f_c.write("    ;\n")

			f_c.write("  }\n\n")

		f_c.write("  return 0;\n}\n")


	def writeEventsAssignmentFunction(self, f_h, f_c, model_id):
		""" Writes the events assignments function in C files """

		f_h.write("int assign_events_%d(realtype t, N_Vector y, void *user_data, int assignment_id, realtype * memory);\n" % model_id)
		f_c.write("int assign_events_%d(realtype t, N_Vector y, void *user_data, int assignment_id, realtype * memory)\n{\n" % model_id)
		if self.getMathModel().listOfEvents.nbValidEvents() > 0:

			f_c.write("  IntegrationData * data = (IntegrationData *) user_data;\n")
			f_c.write("  N_Vector cst = data->constant_variables;\n")
			f_c.write("  N_Vector ass = data->assignment_variables;\n")

			f_c.write("  switch(assignment_id) {\n")

			i_event = 0
			for event in self.getMathModel().listOfEvents.validEvents():

				f_c.write("    case %d :\n" % i_event)

				for i_assignment, event_assignment in enumerate(event.listOfEventAssignments):
					if event_assignment.isValid():
						if event.useValuesFromTriggerTime:
							if event.delay is not None and event.delay.notZero():
								f_c.write("      %s = memory[%d];\n" % (
									event_assignment.getVariable().symbol.getCMathFormula(),
									i_assignment
								))

							else:
								f_c.write("      %s = data->events_memory[%d][%d];\n" % (
									event_assignment.getVariable().symbol.getCMathFormula(),
									i_event, i_assignment
								))

						else:
							# We need to put an empty statement for some weird rule
							# about a declaration not being allowed as first instruction
							f_c.write("      ;\n")
							f_c.write("      realtype t_var_%d_%d = %s;\n" % (
								i_event, i_assignment,
								event_assignment.getDefinition().getCMathFormula()
							))

				for i_assignment, event_assignment in enumerate(event.listOfEventAssignments):
					if event_assignment.isValid() and not event.useValuesFromTriggerTime:
						f_c.write("      %s = t_var_%d_%d;\n" % (
							event_assignment.getVariable().symbol.getCMathFormula(),
							i_event, i_assignment
						))

				f_c.write("      break;\n")
				i_event += 1

			f_c.write("  }\n")

		f_c.write("  return 0;\n}\n")


	def writeEventsPriorityFunction(self, f_h, f_c, model_id):

		f_h.write("int priority_events_%d(realtype t, N_Vector y, void *user_data);\n" % model_id)
		f_c.write("int priority_events_%d(realtype t, N_Vector y, void *user_data)\n{\n" % model_id)
		if self.getMathModel().listOfEvents.nbValidEvents() > 0:

			f_c.write("  IntegrationData * data = (IntegrationData *) user_data;\n")
			f_c.write("  N_Vector cst = data->constant_variables;\n")
			f_c.write("  N_Vector ass = data->assignment_variables;\n")

			for i, event in enumerate(self.getMathModel().listOfEvents.validEvents()):

				if event.priority is not None:
					f_c.write("  *(data->events_priorities[%d]) = %s;\n" % (i, event.priority.getCMathFormula()))

		f_c.write("  return 0;\n}\n")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	Test of combine archive : open a COMBINE archive without manifest. Recreating it

"""

from libsignetsim import CombineArchive, Settings

from unittest import TestCase
from os.path import join, dirname


class TestNoManifest(TestCase):

	def testNoManifest(self):

		ca = CombineArchive()
		ca.readArchive(join(join(dirname(__file__), "files"), "BIOMD0000000003.sedx"))
		ca.writeArchive(join(Settings.tempDirectory, "BIOMD0000000003_fixed.sedx"))

		ca_fixed = CombineArchive()
		ca_fixed.readArchive(join(Settings.tempDirectory, "BIOMD0000000003_fixed.sedx"))

		expected_results = {
			"BIOMD0000000003.sedx.xml": ("http://identifiers.org/combine.specifications/sed-ml.level-1.version-1", True),
			"model1.xml": ("http://identifiers.org/combine.specifications/sbml.level-2.version-4", False)
		}
		for file in ca_fixed.getListOfFiles():
			self.assertEqual(expected_results[file.getFilename()][0], file.getFormat())
			self.assertEqual(expected_results[file.getFilename()][1], file.isMaster())








#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf
from libsignetsim.sedml.AlgorithmParameter import AlgorithmParameter
from libsignetsim.settings.Settings import Settings


class ListOfAlgorithmParameters(ListOf):

	def __init__(self, document):

		ListOf.__init__(self, document)

		self.__document = document
		# self.listOfAlgorithmParameters = []

	def readSedml(self, list_of_algo_params, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_algo_params, level, version)

		for t_algo_param in list_of_algo_params:
			algo_param = AlgorithmParameter(self.__document)
			algo_param.readSedml(t_algo_param, level, version)
			ListOf.append(self, algo_param)

	def writeSedml(self, list_of_algo_params, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_algo_params, level, version)

		for t_algo_param in self:
			algo_param = list_of_algo_params.createAlgorithmParameter()
			t_algo_param.writeSedml(algo_param, level, version)

	def hasRelTol(self):

		for param in self:
			if param.isRelTol():
				return True
		return False

	def hasAbsTol(self):

		for param in self:
			if param.isAbsTol():
				return True
		return False


	def getRelTol(self):

		for param in self:
			if param.isRelTol():
				return float(param.getValue())

	def getAbsTol(self):

		for param in self:
			if param.isAbsTol():
				return float(param.getValue())

	def setRelTol(self, rel_tol):

		param = AlgorithmParameter(self.__document)
		param.setRelTol(rel_tol)
		ListOf.append(self, param)

	def setAbsTol(self, abs_tol):

		param = AlgorithmParameter(self.__document)
		param.setAbsTol(abs_tol)
		ListOf.append(self, param)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.cwriter.CWriterOptimization import CWriterOptimization
from libsignetsim.optimization.OptimizationParameters import OptimizationParameters
from libsignetsim.optimization.OptimizationExecution import OptimizationExecution

from os.path import join, isfile, isdir
from os import mkdir


class Optimization(OptimizationExecution, OptimizationParameters):

	MODEL_VS_MODEL = 0
	MODEL_VS_DATA = 1
	MODEL_VS_STEADYSTATES = 2

	def __init__(self, workingModel=None, parameters_to_fit=None, optimization_type=MODEL_VS_DATA, model_instance=False):

		self.workingModel = workingModel
		self.optimizationType = optimization_type

		OptimizationExecution.__init__(self)
		OptimizationParameters.__init__(self, workingModel, parameters_to_fit, model_instance)


	def writeOptimizationFilesMain(self):

		if not isdir(self.getTempDirectory()):
			mkdir(self.getTempDirectory())

		if self.workingModel.parentDoc is not None:
			self.workingModel.parentDoc.writeSbmlToFile(join(self.getTempDirectory(), "model.sbml"))

		self.workingModel.build(reduce=False)


	def initializeOptimizationParameters(self):

		for constant_type, constant in self.constantsToFit:
			if constant_type == 1:
				self.workingModel.listOfSpecies[constant].init_value = 1e-8
			elif constant_type == 2:
				self.workingModel.listOfParameters[constant].value = 1e-8
			elif constant_type == 3:
				(r_id, lp_id) = constant
				self.workingModel.listOfReactions[r_id].listOfLocalParameters[lp_id].value = 1e-8


	def getBestResult(self, nb_procs):

		if isfile(join(self.getTempDirectory(), "logs/score/score")):
			f_best_score = open(join(self.getTempDirectory(), "logs/score/score"), 'r')
			t_score = f_best_score.readline()
			if t_score is not None and t_score != "":
				res = float(t_score)
			else:
				return (-1, 1e+100)
			f_best_score.close()

			return (0, res)

		return (-1,1e+100)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.SimpleSbmlObject import SimpleSbmlObject

from libsignetsim.model.sbml.ReplacedElement import ReplacedElement
from libsignetsim.settings.Settings import Settings

class ListOfReplacedElements(ListOf):#, SimpleSbmlObject):
	""" Class for the listOfModelDefinition in a sbml model """

	def __init__ (self, model, parent_obj):

		self.__model = model
		self.__parentObj = parent_obj
		ListOf.__init__(self, model)
		# SimpleSbmlObject.__init__(self, model)


	def readSbml(self, sbml_list_re,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Reads compartments' list from a sbml file """

		for re in sbml_list_re:
			t_re = ReplacedElement(self.__model, self.__parentObj, self.nextId())
			t_re.readSbml(re, sbml_level, sbml_version)
			ListOf.add(self, t_re)


	def writeSbml(self, sbml_object,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Writes compartments' list to a sbml file """

		for t_replaced_element in self:
			t_replaced_element.writeSbml(sbml_object, sbml_level, sbml_version)

	def new(self):
		""" Creates a new compartment """

		t_replaced_element = ReplacedElement(self.__model, self.__parentObj, self.nextId())
		ListOf.add(self, t_replaced_element)
		return t_replaced_element

	def copy(self, obj, prefix="", shift=0):

		for replaced_element in obj:
			t_re = ReplacedElement(self.__model, self.__parentObj, (replaced_element.objId + shift))
			t_re.copy(replaced_element, prefix, shift)
			ListOf.add(self, t_re)

	def remove(self, obj):
		""" 
			Remove an object from the list 
			Using list base method to avoid having to remove it from the listOfSbmlObjects (it's not there) 
		"""
		list.remove(self, obj)

	def getByReplacedElementObject(self, re_object):

		for replacement_element in self:
			if replacement_element.getReplacedElementObject() == re_object:
				return replacement_element

	def containsSubmodel(self, submodel_ref):

		for replaced_element in self:
			if replaced_element.getSubmodelRef() == submodel_ref:
				return True

		return False


#!/usr/bin/env python
""" TestParameterScan1D.py


	Test of combine archive : run all sedml files


	Copyright (C) 2016 Vincent Noel (vincent.noel@butantan.gov.br)

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.

"""

from libsignetsim.settings.Settings import Settings
from libsignetsim.combine.CombineArchive import CombineArchive
from unittest import TestCase
from os.path import join, dirname, isdir
from os import mkdir

class TestParameterScan1D(TestCase):

	expected_results = {}

	def test(self):

		ca = CombineArchive()
		ca.readArchive(join(join(dirname(__file__), "files"), "parameterScan1D.sedx"))
		sedml_doc = ca.runMasterSedml()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf
from libsignetsim.sedml.Variable import Variable
from libsignetsim.settings.Settings import Settings


class ListOfVariables(ListOf):

	def __init__(self, document, parent):

		ListOf.__init__(self, document)

		self.__document = document
		self.__parent = parent
		self.__variableCounter = 0

	def new(self, variable_id=None):

		t_variable_id = variable_id
		if t_variable_id is None:
			t_variable_id = "%s_variable_%d" % (self.__parent.getId(), self.__variableCounter)

		variable = Variable(self.__document)
		variable.setId(t_variable_id)
		self.__variableCounter += 1
		ListOf.append(self, variable)
		return variable

	def createVariable(self, variable_id=None):
		return self.new(variable_id)

	def readSedml(self, list_of_variables, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_variables, level, version)

		for t_var in list_of_variables:
			var = Variable(self.__document)
			var.readSedml(t_var, level, version)
			ListOf.append(self, var)
			self.__variableCounter += 1

	def writeSedml(self, list_of_variables, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_variables, level, version)

		for t_var in self:
			var = list_of_variables.createVariable()
			t_var.writeSedml(var, level, version)

	def getData(self):

		data = {}
		for variable in self:
			data.update(variable.getData())

		return data

	def getTasksToRun(self):

		tasks = []
		for variable in self:
			tasks.append(variable.getTask())

		return list(set(tasks))


#!/usr/bin/env python
""" TestSuite.py


	This file is made for the test cases from the SED-ML files
	in the SBmL Test Suite

	Copyright (C) 2016 Vincent Noel (vincent.noel@butantan.gov.br)

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.

"""

from libsignetsim.tests.sbmltestsuite.TestSuite import TestSuite
from unittest import TestCase


class TestSuite_l3v2(TestCase):
	""" Tests SED-ML semantic test cases """

	def __init__(self, *args, **kwargs):

		TestCase.__init__(self, *args, **kwargs)

		self.testSuite = TestSuite('3.2')

	def testSuiteRun(self):

		self.assertTrue(self.testSuite.testSuiteRun())




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.container.ListOf import ListOf
from libsignetsim.model.sbml.container.HasIds import HasIds
from libsignetsim.model.ModelException import CannotDeleteException, InvalidXPath
from libsignetsim.model.sbml.SubModel import SubModel
from libsignetsim.settings.Settings import Settings
from re import match


class ListOfSubmodels(ListOf, HasIds):#, SbmlObject):
	""" Class for the listOfModelDefinition in a sbml model """

	def __init__ (self, model):

		self.__model = model
		ListOf.__init__(self, model)
		HasIds.__init__(self, model)
		# SbmlObject.__init__(self, model)


	def readSbml(self, sbml_list_submodels,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Reads compartments' list from a sbml file """

		for i, submodel in enumerate(sbml_list_submodels):
			t_submodel = SubModel(self.__model, self.nextId())
			t_submodel.readSbml(submodel, sbml_level, sbml_version)
			ListOf.add(self, t_submodel)

		# SbmlObject.readSbml(self, sbml_list_submodels, sbml_level, sbml_version)


	def writeSbml(self, sbml_model,
					sbml_level=Settings.defaultSbmlLevel,
					sbml_version=Settings.defaultSbmlVersion):
		""" Writes compartments' list to a sbml file """

		for t_submodel in self:
			t_submodel.writeSbml(sbml_model, sbml_level, sbml_version)

		# SbmlObject.writeSbml(self, sbml_model, sbml_level, sbml_version)


	def new(self):
		""" Creates a new compartment """

		t_submodel = SubModel(self.__model, self.nextId())
		ListOf.add(self, t_submodel)
		return t_submodel


	def remove(self, sbml_obj):
		""" Remove an object from the list """

		for object in self.__model.listOfSbmlObjects:
			if object.hasReplacedElements() and object.getListOfReplacedElements().containsSubmodel(sbml_obj.getSbmlId()):
				raise CannotDeleteException("Submodel %s is used. Please remove the substitutions first" % sbml_obj.getNameOrSbmlId())

		list.remove(self, sbml_obj)

	def removeById(self, obj_id):
		""" Remove an object from the list """
		self.remove(self.getById(obj_id))

	def getSubmodels(self):

		res = []
		for submodel in self:
			res.append(submodel.getModelObject())

		return res

	def getBySbmlId(self, sbml_id_ref):
		return HasIds.getBySbmlId(self, sbml_id_ref)

	def getBySbmlIdRef(self, sbml_id):
		for submodel in self:
			if submodel.getModelObject().getSbmlId() == sbml_id:
				return submodel

	def resolveXPath(self, selector):

		if not (selector.startswith("submodel") or selector.startswith("sbml:submodel")):
			raise InvalidXPath(selector)

		res_match = match(r'(.*)\[@(.*)=\'(.*)\'\]', selector)
		if res_match is None:
			raise InvalidXPath(selector)


		tokens = res_match.groups()
		if len(tokens) != 3:
			raise InvalidXPath(selector)

		object = None
		if tokens[1] == "id":
			object = HasIds.getBySbmlId(self, tokens[2])
		elif tokens[1] == "name":
			object = HasIds.getByName(self, tokens[2])

		if object is not None:
			return object

		# If not returned yet
		raise InvalidXPath(selector)

	def getByXPath(self, xpath):
		return self.resolveXPath(xpath[0]).getByXPath(xpath[1:])

	def setByXPath(self, xpath, object):
		self.resolveXPath(xpath[0]).setByXPath(xpath[1:], object)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file is a simple example of reading and writing a basic NuML doc

"""
from __future__ import print_function
import libnuml
from libnuml import readNUMLFromFile, writeNUML, writeNUMLToString, XMLNode, NUMLDocument
from libsignetsim import Settings
from unittest import TestCase
from os.path import join, dirname
from six.moves import reload_module


class TestExampleNotes(TestCase):
	""" Tests high level functions """

	def testExampleNotes(self):

		print("\n\n")
		numl_doc = NUMLDocument()
		reload_module(libnuml)
		time_term = numl_doc.createOntologyTerm()
		time_term.setId("time_term")
		time_term.setTerm("time")
		time_term.setSourceTermId("SBO:0000345")
		time_term.setOntologyURI("http://www.ebi.ac.uk/sbo/")
		notes = "<notes><body xmlns=\"http://www.w3.org/1999/xhtml\"><p>This needs to be noted</p></body></notes>"
		xml_notes = XMLNode.convertStringToXMLNode(notes)
		numl_doc.setNotes(xml_notes)

		numl_doc_string = writeNUMLToString(numl_doc)
		numl_doc_filename = join(Settings.tempDirectory, "example_notes.xml")
		writeNUML(numl_doc, numl_doc_filename)

		numl_doc_copy_file = readNUMLFromFile(numl_doc_filename)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.sedml.container.ListOf import ListOf
from libsignetsim.sedml.DataGenerator import DataGenerator
from libsignetsim.settings.Settings import Settings


class ListOfDataGenerators(ListOf):

	def __init__(self, document):

		ListOf.__init__(self, document)
		self.__document = document
		self.__dataGeneratorCounter = 0

	def new(self, data_generator_id=None):

		t_data_generator_id = data_generator_id
		if t_data_generator_id is None:
			t_data_generator_id = "datagenerator_%d" % self.__dataGeneratorCounter

		data_generator = DataGenerator(self.__document)
		data_generator.setId(t_data_generator_id)
		ListOf.append(self, data_generator)
		self.__dataGeneratorCounter += 1
		return data_generator


	def createDataGenerator(self, data_generator_id=None):
		return self.new(data_generator_id)


	def readSedml(self, list_of_data_generators, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.readSedml(self, list_of_data_generators, level, version)

		for t_data_generator in list_of_data_generators:

			data_generator = DataGenerator(self.__document)
			data_generator.readSedml(t_data_generator, level, version)
			ListOf.append(self, data_generator)
			self.__dataGeneratorCounter += 1

	def writeSedml(self, list_of_data_generators, level=Settings.defaultSedmlLevel, version=Settings.defaultSedmlVersion):

		ListOf.writeSedml(self, list_of_data_generators, level, version)

		for t_data_generator in self:
			data_generator = list_of_data_generators.createDataGenerator()
			t_data_generator.writeSedml(data_generator, level, version)

	def getDataGenerator(self, data_reference):

		for data_generator in self:
			if data_generator.getId() == data_reference:
				return data_generator

	def getTasksToRun(self, data_to_generate):

		tasks = []
		for data in data_to_generate:
			tasks += data.getTasksToRun()

		return list(set(tasks))

	def build(self):
		for data_genertor in self:
			data_genertor.build()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""
from __future__ import print_function


from libsignetsim.model.math.MathFormula import MathFormula
from libsignetsim.model.math.ConservationLaw import ConservationLaw
from libsignetsim.model.math.sympy_shortcuts import SympySymbol, SympyInteger, SympyEqual, SympyAdd, SympyMul, SympyPow
from libsignetsim.model.ListOfVariables import ListOfVariables
from libsignetsim.model.sbml.ConservedMoiety import ConservedMoiety
from libsignetsim.model.math.MathDevelopper import unevaluatedSubs

from sympy import ones, pretty, srepr


class ListOfConservationLaws(list):
	""" Sbml model class """

	def __init__(self, model):
		""" Constructor of model class """

		list.__init__(self)
		self.__model = model

		self.extraVariables = ListOfVariables(self.__model)
		self.conservationMatrix = None
		self.__upToDate = False
	def getRawFormulas(self, stoichiometry_matrix):
		return [SympyEqual(law, value) for law, value in self.__build(stoichiometry_matrix)]

	def isUpToDate(self):
		return self.__upToDate

	def build(self):

		self.clear()
		laws = self.__build()
		for law, value in laws:
			self.__buildConservationLaw(law, value)

		self.__upToDate = True

	def __str__(self):
		res = ""
		for law in self:
			res += "%s\n" % law
		return res

	def pprint(self):

		for cons in self:
			cons.pprint()
			print("\n")

	def hasConservationLaws(self):
		return len(self) > 0

	def clear(self):
		del self[:]
		to_remove = []

		for variable in self.__model.listOfVariables:
			if variable.isConservedMoiety():
				to_remove.append(variable)
				if variable.isConstant():
					self.__model.variablesConstant.remove(variable)
					self.__model.nbConstants -= 1

		for variable in to_remove:
			self.__model.listOfVariables.remove(variable)

	def __getVariableFormula(self, variable):

		formula = variable.symbol.getSymbol()

		if variable.isSpecies():
			if variable.isSetConversionFactor():
				formula /= variable.getSymbolConversionFactor()

			elif self.__model.isSetConversionFactor():
				formula /= self.__model.getSymbolConversionFactor()

		elif self.__model.isSetConversionFactor():
			formula /= self.__model.getSymbolConversionFactor()

		return formula

	def __getVariableValue(self, variable):

		symbol = variable.symbol.getSymbol()

		if symbol in list(self.__model.listOfInitialConditions.keys()):

			value = self.__model.listOfInitialConditions[symbol].getDeveloppedInternalMathFormula()
			if variable.isSpecies():
				if variable.isSetConversionFactor():
					conv_factor = variable.getSymbolConversionFactor()
					value /= self.__model.listOfInitialConditions[conv_factor].getDeveloppedInternalMathFormula()

				elif self.__model.isSetConversionFactor():
					conv_factor = self.__model.getSymbolConversionFactor()
					value /= self.__model.listOfInitialConditions[conv_factor].getDeveloppedInternalMathFormula()
			elif self.__model.isSetConversionFactor():
				conv_factor = self.__model.getSymbolConversionFactor()
				value /= self.__model.listOfInitialConditions[conv_factor].getDeveloppedInternalMathFormula()

		else:
			value = SympySymbol("_%s_0_" % str(symbol))

		return value

	def __addConservedMoiety(self, value):

		new_var = ConservedMoiety(self.__model)
		name = "total_%d" % len(self)
		new_var.new(name, value)
		self.__solveInitialConditions(new_var)

		return new_var.symbol.getInternalMathFormula(rawFormula=True)

	def __solveInitialConditions(self, moiety):

		solved_value = MathFormula(self.__model)
		t_solved_value = moiety.value.getInternalMathFormula()

		for var in t_solved_value.atoms(SympySymbol):
			if var not in [SympySymbol("_avogadro_")]:
				t_solved_value = unevaluatedSubs(
					t_solved_value,
					{var: self.__model.listOfInitialConditions[var].getInternalMathFormula()})

		solved_value.setInternalMathFormula(t_solved_value)
		self.__model.listOfInitialConditions.update({moiety.symbol.getSymbol(): solved_value})

	def __buildConservationLaw(self, t_law, t_value):

		t_vars = []
		for t_atom in t_law.atoms(SympySymbol):

			if not self.__model.listOfVariables.getBySymbol(t_atom).isCompartment():
				t_vars.append(t_atom)

		t_lhs = MathFormula(self.__model)
		t_lhs.setInternalMathFormula(t_law)

		final_value = SympyInteger(0)

		if t_value.func == SympyAdd:
			# Should be only positive terms, divided by the compartment
			for arg in t_value.args:
				moeity_value = self.__addConservedMoiety(arg)
				final_value += moeity_value
		else:
			final_value = self.__addConservedMoiety(t_value)

		t_rhs = MathFormula(self.__model)
		t_rhs.setInternalMathFormula(final_value)

		t_conservation_law = ConservationLaw(self.__model)
		t_conservation_law.new(t_lhs, t_rhs, t_vars)
		list.append(self, t_conservation_law)

	def __build(self, stoichiometry_matrix=None):

		laws = []
		if not self.__model.listOfReactions.hasVariableStoichiometry():

			conservation_matrix = self.__model.conservationMatrix.getConservationMatrix(stoichiometry_matrix)
			if conservation_matrix is not None:
				for i in range(conservation_matrix.shape[0]):

					t_res = conservation_matrix[i, :]

					t_law = MathFormula.ZERO
					t_value = MathFormula.ZERO

					nb_vars_found = t_res * ones(conservation_matrix.shape[1], 1)

					if int(nb_vars_found[0, 0]) > 1:

						for ii, tt_res in enumerate(t_res):

							variable = self.__model.variablesOdes[ii]
							tt_symbol_formula = self.__getVariableFormula(variable)
							tt_value = self.__getVariableValue(variable)

							if tt_res == SympyInteger(1):
								t_law += tt_symbol_formula
								t_value += tt_value

							elif tt_res == SympyInteger(-1):
								t_law -= tt_symbol_formula
								t_value -= tt_value

							else:
								t_law += tt_res * tt_symbol_formula
								t_value += tt_res * tt_value

						laws.append((t_law, t_value))
		return laws



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

    This file ...

"""
from __future__ import print_function

from libsignetsim.simulation.Simulation import Simulation
from libsignetsim.data.Experiment import Experiment

from libsignetsim.settings.Settings import Settings

from os.path import join, isfile
from time import clock


class SteadyStatesSimulation(Simulation):

    SIM_DONE            =   10
    SIM_TODO            =   11

    def __init__(self,
                 list_of_models=[],
                 species_input=None,
                 list_of_initial_values=[],
                 time_max=Settings.steadyStatesMaxTime,
                 keep_files=Settings.simulationKeepFiles):

        self.listOfInitialValues = list_of_initial_values
        self.experiment = None
        self.generateExperiment(species_input, list_of_initial_values)

        Simulation.__init__(self,
            list_of_models=list_of_models,
            time_min=0,
            list_samples=[0.0, time_max*1.1],
            experiment=self.experiment,
            keep_files=keep_files
        )

    def generateExperiment(self, species_input, list_of_initial_values):

        if len(list_of_initial_values) > 0:

            self.experiment = Experiment()
            self.experiment.name = "SteadyState"
            for initial_value in list_of_initial_values:

                t_condition = self.experiment.createCondition()
                t_condition.addInitialCondition(
                    t=0, name=species_input.getSbmlId(),
                    value=initial_value, name_attribute="id"
                )

    def run(self, timeout=None):

        start = clock()
        self.writeSimulationFiles()
        mid = clock()

        if Settings.verboseTiming >= 1:
            print("> Files written in %.2fs" % (mid-start))

        self.runSimulation(steady_states=True, timeout=timeout)
        self.loadSimulationResults()

        if not self.keepFiles:
            self.cleanTempDirectory()

        stop = clock()

        if Settings.verboseTiming >= 1:
            print("> Simulation executed in %.2fs" % (stop-start))

    def loadSimulationResults(self):

        t_model = self.listOfModels[0]
        self.rawData = {}

        for variable in t_model.listOfVariables:
            self.rawData.update({variable.getSbmlId(): []})

        if len(self.listOfInitialValues) > 1:

            for i_initial_values, _ in enumerate(self.listOfInitialValues):

                t_file = join(
                    self.getTempDirectory(),
                    Settings.C_simulationResultsDirectory,
                    "results_0_%d" % i_initial_values
                )

                if isfile(t_file):
                    self.readFile(t_file)

        else:
            t_file = join(
                self.getTempDirectory(),
                Settings.C_simulationResultsDirectory,
                "results_0"
            )

            if isfile(t_file):
                self.readFile(t_file)

    def readFile(self, filename):
        resultsFile = open(filename, "r")
        variables = []
        for i, line in enumerate(resultsFile):
            data = line.split()

            if i == 0:
                variables = data
            else:
                for i_value, value in enumerate(data):
                    f_value = float(value)
                    array = self.rawData[variables[i_value]]
                    array.append(f_value)
                    self.rawData.update({variables[i_value]: array})

        resultsFile.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.model.sbml.EventAssignedVariable import EventAssignedVariable
from libsignetsim.model.sbml.InitiallyAssignedVariable import InitiallyAssignedVariable
from libsignetsim.model.sbml.RuledVariable import RuledVariable
from libsignetsim.model.Variable import Variable
from libsignetsim.model.sbml.SbmlObject import SbmlObject
from libsignetsim.model.sbml.HasUnits import HasUnits


class ConservedMoiety(Variable, SbmlObject, InitiallyAssignedVariable,
						RuledVariable, EventAssignedVariable,
						HasUnits):

	def __init__(self, model):

		SbmlObject.__init__(self, model)
		InitiallyAssignedVariable.__init__(self, model)
		RuledVariable.__init__(self, model)
		EventAssignedVariable.__init__(self, model)
		HasUnits.__init__(self, model)
		Variable.__init__(self, model, Variable.CONSERVED_MOIETY)

		self.__model = model

	def copy(self, parameter, sids_subs={}, symbols_subs={}, usids_subs={}, conversion_factor=None):

		SbmlObject.copy(self, parameter)
		HasUnits.copy(self, parameter, usids_subs=usids_subs)
		Variable.copy(self, parameter, sids_subs=sids_subs, symbols_subs=symbols_subs, conversion_factor=conversion_factor)

	def new(self, name, value, unit=None):
		""" Creates new compartment with default options """

		SbmlObject.new(self)
		Variable.new(self, name, Variable.CONSERVED_MOIETY)
		HasUnits.new(self, unit)

		self.value.setInternalMathFormula(value)
		self.__model.listOfVariables.changeVariableType(self, Variable.VAR_CST)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


from libsignetsim.settings.Settings import Settings
import libsbml
from libnuml import XMLNode
from six.moves import reload_module
reload_module(libsbml)


class NMBase (object):

	def __init__(self, document):

		self.__document = document
		self.__metaId = None
		self.__annotation = None
		self.__notes = None


	def readNuML(self, object, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):

		if object.isSetMetaId():
			self.__metaId = object.getMetaId()
		#
		# if object.isSetAnnotation():
		# 	self.__annotation = object.getAnnotation()

		if object.isSetNotes():
			self.__notes = XMLNode.convertXMLNodeToString(object.getNotes().getChild(0).getChild(0).getChild(0))

	def writeNuML(self, object, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):

		if self.__metaId is not None:
			object.setMetaId(self.__metaId)

		# if self.__annotation is not None:
		# 	object.setAnnotation(self.__annotation)

		if self.__notes is not None:
			xml_string = "<notes><body xmlns=\"http://www.w3.org/1999/xhtml\">%s</body></notes>" % self.__notes
			xml_node = XMLNode.convertStringToXMLNode(xml_string)
			object.setNotes(xml_node)


	def getNotes(self):
		return self.__notes

	def setNotes(self, notes=""):
		self.__notes = notes




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""

from libsignetsim.numl.DimensionDescription import DimensionDescription
from libsignetsim.settings.Settings import Settings

class AtomicDescription (DimensionDescription):

	def __init__(self, document, name=None, value_type="double"):
		DimensionDescription.__init__(self, document, name)
		self.__document = document
		self.__valueType = value_type
		self.__ontologyTerm = None

	def readNuML(self, atomic_description, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):

		DimensionDescription.readNuML(self, atomic_description, level, version)
		self.__valueType = atomic_description.getValueType()
		self.__ontologyTerm = self.__document.listOfOntologyTerms.getByOntologyTerm(atomic_description.getOntologyTerm())

	def writeNuML(self, atomic_description, level=Settings.defaultNuMLLevel, version=Settings.defaultNuMLVersion):

		DimensionDescription.writeNuML(self, atomic_description, level, version)
		if self.__valueType is not None:
			atomic_description.setValueType(self.__valueType)

		if self.__ontologyTerm is not None:
			atomic_description.setOntologyTerm(self.__ontologyTerm.getId())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


import re

class ExperimentalData(object):

	def __init__(self, t=0, name="", value=0, value_dev=None, name_attribute="name", quantification_ratio=1):

		self.name = name
		self.t = t
		self.value = value
		self.value_dev = value_dev
		self.name_attribute = name_attribute
		self.quantification_ratio = quantification_ratio

		self.steady_state = False
		self.min_steady_state = 0
		self.max_steady_state = 0


		# Not used ?
		self.variableName = ""
		self.variableId = None

	def readNuML(self, composite_value):
		self.t = float(composite_value.getIndexValue())
		res = re.match(
			r"/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species\[@(.*)=\'(.*)\'\]",
			composite_value.getContents()[0].getIndexValue()
		)
		self.name_attribute = res.groups()[0]
		self.name = res.groups()[1]
		tuple = composite_value.getContents()[0].getContents()[0].getAtomicValues()

		self.value = float(tuple[0].getValue())

		if tuple[1].getValue() is not None:
			self.value_dev = float(tuple[1].getValue())

	def writeNuML(self, composite_value):

		time_desc = composite_value.getDescription().getContent()
		time_index = composite_value.createCompositeValue(time_desc, self.t)

		species_desc = time_desc.getContent()
		species_index = time_index.createCompositeValue(
			species_desc,
			"/sbml:sbml/sbml:model/sbml:listOfSpecies/sbml:species[@%s='%s']" % (self.name_attribute, self.name)
		)

		tuple_desc = species_desc.getContent()
		tuple = species_index.createTupleValue(tuple_desc)

		value_desc = tuple_desc.getAtomicDescriptions()[0]
		value = tuple.createAtomicValue(
			value_desc,
			str(self.value)
		)

		std_desc = tuple_desc.getAtomicDescriptions()[1]
		if self.value_dev is not None:
			std = tuple.createAtomicValue(
				std_desc,
				str(self.value_dev)
			)
		else:
			std = tuple.createAtomicValue(
				std_desc,
				""
			)

	# def readDB(self, name, time, value, value_dev=0, steady_state=False,
	# 			min_steady_state=0, max_steady_state=0, quantification_ratio=1):
	#
	# 	self.name = name
	# 	self.t = time
	# 	self.value = value
	# 	self.value_dev = value_dev
	# 	self.steady_state = steady_state
	# 	if steady_state:
	# 		self.min_steady_state = min_steady_state
	# 		self.max_steady_state = max_steady_state
	# 	self.quantification_ratio = quantification_ratio



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014-2017 Vincent Noel (vincent.noel@butantan.gov.br)
#
# This file is part of libSigNetSim.
#
# libSigNetSim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libSigNetSim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libSigNetSim.  If not, see <http://www.gnu.org/licenses/>.

"""

	This file ...

"""


class HasMetaIds(object):
	""" Parent class for all the ListOf containers in a sbml model """

	def __init__ (self, model=None):

		self.__model = model




	def metaIds(self):
		""" Return set of names of the sbml objects """
		return [obj.getMetaId() for obj in self]

	def getByMetaId(self, meta_id, pos=0):
		""" Find sbml objects by their name """

		res = []
		for obj in self:
			if obj.getMetaId() == meta_id:
				res.append(obj)

		if len(res) > 0:
			return res[pos]
		else:
			return None


	def containsMetaId(self, meta_id):
		""" Test if a name is in the list """

		res = False
		for obj in self:
			if meta_id == obj.getMetaId():
				res = True

		return res
